@@ -20,42 +20,42 @@
 #include "template.h"
 #include <qlist.h>
 #include <stdio.h>
-#include "dirdef.h"
+#include "classdef.h"
+#include "searchindex.h"
+#include "membergroup.h"
 
 class Definition;
-class ClassDef;
-class ClassSDict;
-class BaseClassList;
+
 class PageDef;
+class PageLinkedMap;
+class PageLinkedRefMap;
+
 class GroupDef;
+class GroupLinkedMap;
+class GroupList;
+
 class NamespaceDef;
-class BaseClassList;
-class NamespaceSDict;
+class NamespaceLinkedMap;
+class NamespaceLinkedRefMap;
+
+class FileNameLinkedMap;
+class ClassLinkedMap;
+class MemberNameInfoLinkedMap;
+
+class DirDef;
+class DirLinkedMap;
+class DirList;
+
 class FileDef;
 class FileList;
-class FileNameLinkedMap;
-class DirSDict;
-class DirDef;
-class PageSDict;
-class GroupSDict;
-class GroupDef;
-class GroupList;
 struct IncludeInfo;
 class MemberList;
-class MemberSDict;
 class MemberDef;
 struct Argument;
 class ArgumentList;
-class MemberNameInfoLinkedMap;
 class MemberInfo;
-class MemberGroup;
-class MemberGroupSDict;
-class MemberGroupList;
 class DotNode;
 class DotGfxHierarchyTable;
-struct SearchIndexInfo;
-class SearchIndexList;
-class SearchDefinitionList;
 
 //----------------------------------------------------
 
@@ -455,7 +455,7 @@ class ClassInheritanceNodeContext : public RefCountedContext, public TemplateStr
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
 
-    void addChildren(const BaseClassList *bcl,bool hideSuper);
+    void addChildren(const BaseClassList &bcl,bool hideSuper);
 
   private:
     ClassInheritanceNodeContext(const ClassDef *);
@@ -510,8 +510,9 @@ class NestingNodeContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
     static NestingNodeContext *alloc(const NestingNodeContext *parent,const Definition *def,
-                                     int index,int level,bool addClasses,bool inherit,bool hideSuper)
-    { return new NestingNodeContext(parent,def,index,level,addClasses,inherit,hideSuper); }
+                                     int index,int level,bool addClasses,bool inherit,bool hideSuper,
+                                     ClassDefSet &visitedClasses)
+    { return new NestingNodeContext(parent,def,index,level,addClasses,inherit,hideSuper,visitedClasses); }
 
     QCString id() const;
 
@@ -522,7 +523,8 @@ class NestingNodeContext : public RefCountedContext, public TemplateStructIntf
 
   private:
     NestingNodeContext(const NestingNodeContext *parent,
-                       const Definition *,int index,int level,bool addClasses,bool inherit,bool hideSuper);
+                       const Definition *,int index,int level,bool addClasses,bool inherit,bool hideSuper,
+                       ClassDefSet &visitedClasses);
    ~NestingNodeContext();
     class Private;
     Private *p;
@@ -533,6 +535,7 @@ class NestingNodeContext : public RefCountedContext, public TemplateStructIntf
 class NestingContext : public RefCountedContext, public TemplateListIntf
 {
   public:
+
     static NestingContext *alloc(const NestingNodeContext *parent,int level)
     { return new NestingContext(parent,level); }
 
@@ -543,17 +546,20 @@ class NestingContext : public RefCountedContext, public TemplateListIntf
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
 
-    void addNamespaces(const NamespaceSDict &nsDict,bool rootOnly,bool addClasses);
-    void addClasses(const ClassSDict &clDict,bool rootOnly);
-    void addDirs(const DirSDict &);
-    void addDirs(const DirList &);
-    void addFiles(const FileNameLinkedMap &);
-    void addFiles(const FileList &);
-    void addPages(const PageSDict &pages,bool rootOnly);
-    void addModules(const GroupSDict &modules);
-    void addModules(const GroupList &modules);
-    void addClassHierarchy(const ClassSDict &clDict,bool rootOnly);
-    void addDerivedClasses(const BaseClassList *bcl,bool hideSuper);
+    void addNamespaces(const NamespaceLinkedMap &nsLinkedMap,bool rootOnly,bool addClasses,ClassDefSet &visitedClasses);
+    void addNamespaces(const NamespaceLinkedRefMap &nsLinkedMap,bool rootOnly,bool addClasses,ClassDefSet &visitedClasses);
+    void addClasses(const ClassLinkedMap &clLinkedMap,bool rootOnly,ClassDefSet &visitedClasses);
+    void addClasses(const ClassLinkedRefMap &clLinkedMap,bool rootOnly,ClassDefSet &visitedClasses);
+    void addDirs(const DirLinkedMap &,ClassDefSet &visitedClasses);
+    void addDirs(const DirList &,ClassDefSet &visitedClasses);
+    void addFiles(const FileNameLinkedMap &,ClassDefSet &visitedClasses);
+    void addFiles(const FileList &,ClassDefSet &visitedClasses);
+    void addPages(const PageLinkedMap &pages,bool rootOnly,ClassDefSet &visitedClasses);
+    void addPages(const PageLinkedRefMap &pages,bool rootOnly,ClassDefSet &visitedClasses);
+    void addModules(const GroupLinkedMap &modules,ClassDefSet &visitedClasses);
+    void addModules(const GroupList &modules,ClassDefSet &visitedClasses);
+    void addClassHierarchy(const ClassLinkedMap &clLinkedMap,ClassDefSet &visitedClasses);
+    void addDerivedClasses(const BaseClassList &bcl,bool hideSuper,ClassDefSet &visitedClasses);
 
   private:
     NestingContext(const NestingNodeContext *parent,int level);
@@ -687,7 +693,7 @@ class FileTreeContext : public RefCountedContext, public TemplateStructIntf
 class PageListContext : public RefCountedContext, public TemplateListIntf
 {
   public:
-    static PageListContext *alloc(const PageSDict *pages) { return new PageListContext(pages); }
+    static PageListContext *alloc(const PageLinkedMap &pages) { return new PageListContext(pages); }
 
     // TemplateListIntf methods
     virtual uint count() const;
@@ -696,10 +702,10 @@ class PageListContext : public RefCountedContext, public TemplateListIntf
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
 
-    void addPages(const PageSDict &pages);
+    void addPages(const PageLinkedMap &pages);
 
   private:
-    PageListContext(const PageSDict *pages);
+    PageListContext(const PageLinkedMap &pages);
    ~PageListContext();
     class Private;
     Private *p;
@@ -710,7 +716,7 @@ class PageListContext : public RefCountedContext, public TemplateListIntf
 class PageTreeContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static PageTreeContext *alloc(const PageSDict *pages) { return new PageTreeContext(pages); }
+    static PageTreeContext *alloc(const PageLinkedMap &pages) { return new PageTreeContext(pages); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -718,7 +724,7 @@ class PageTreeContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    PageTreeContext(const PageSDict *pages);
+    PageTreeContext(const PageLinkedMap &pages);
    ~PageTreeContext();
     class Private;
     Private *p;
@@ -757,7 +763,7 @@ class ModuleListContext : public RefCountedContext, public TemplateListIntf
     virtual int addRef()  { return RefCountedContext::addRef(); }
     virtual int release() { return RefCountedContext::release(); }
 
-    void addModules(const GroupSDict &);
+    void addModules(const GroupLinkedMap &);
     void addModules(const GroupList &);
 
   private:
@@ -929,7 +935,7 @@ class InheritanceNodeContext : public RefCountedContext, public TemplateStructIn
 class InheritanceListContext : public RefCountedContext, public TemplateListIntf
 {
   public:
-    static InheritanceListContext *alloc(const BaseClassList *list,bool baseClasses)
+    static InheritanceListContext *alloc(const BaseClassList &list,bool baseClasses)
     { return new InheritanceListContext(list,baseClasses); }
 
     // TemplateListIntf
@@ -940,7 +946,7 @@ class InheritanceListContext : public RefCountedContext, public TemplateListIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    InheritanceListContext(const BaseClassList *list,bool baseClasses);
+    InheritanceListContext(const BaseClassList &list,bool baseClasses);
    ~InheritanceListContext();
     class Private;
     Private *p;
@@ -955,8 +961,8 @@ class MemberListContext : public RefCountedContext, public TemplateListIntf
     { return new MemberListContext; }
     static MemberListContext *alloc(const MemberList *ml)
     { return new MemberListContext(ml); }
-    static MemberListContext *alloc(MemberSDict *ml,bool doSort)
-    { return new MemberListContext(ml,doSort); }
+    static MemberListContext *alloc(std::vector<const MemberDef *> &&ml)
+    { return new MemberListContext(std::move(ml)); }
 
     // TemplateListIntf
     virtual uint count() const;
@@ -968,7 +974,7 @@ class MemberListContext : public RefCountedContext, public TemplateListIntf
   private:
     MemberListContext();
     MemberListContext(const MemberList *ml);
-    MemberListContext(MemberSDict *ml,bool doSort);
+    MemberListContext(std::vector<const MemberDef *> &&ml);
    ~MemberListContext();
     class Private;
     Private *p;
@@ -1001,10 +1007,10 @@ class MemberGroupListContext : public RefCountedContext, public TemplateListIntf
   public:
     static MemberGroupListContext *alloc()
     { return new MemberGroupListContext; }
-    static MemberGroupListContext *alloc(const Definition *def,const QCString &relPath,const MemberGroupList *list)
+    static MemberGroupListContext *alloc(const Definition *def,const QCString &relPath,const MemberGroupRefList &list)
     { return new MemberGroupListContext(def,relPath,list); }
-    static MemberGroupListContext *alloc(const Definition *def,const QCString &relPath,const MemberGroupSDict *dict,bool subGrouping)
-    { return new MemberGroupListContext(def,relPath,dict,subGrouping); }
+    static MemberGroupListContext *alloc(const Definition *def,const QCString &relPath,const MemberGroupList &list,bool subGrouping)
+    { return new MemberGroupListContext(def,relPath,list,subGrouping); }
 
     // TemplateListIntf
     virtual uint count() const;
@@ -1015,8 +1021,8 @@ class MemberGroupListContext : public RefCountedContext, public TemplateListIntf
 
   private:
     MemberGroupListContext();
-    MemberGroupListContext(const Definition *def,const QCString &relPath,const MemberGroupList *list);
-    MemberGroupListContext(const Definition *def,const QCString &relPath,const MemberGroupSDict *mgDict,bool subGrouping);
+    MemberGroupListContext(const Definition *def,const QCString &relPath,const MemberGroupRefList &list);
+    MemberGroupListContext(const Definition *def,const QCString &relPath,const MemberGroupList &list,bool subGrouping);
    ~MemberGroupListContext();
     class Private;
     Private *p;
@@ -1199,8 +1205,9 @@ class SymbolContext : public RefCountedContext, public TemplateStructIntf
 class SymbolListContext : public RefCountedContext, public TemplateListIntf
 {
   public:
-    static SymbolListContext *alloc(const SearchDefinitionList *sdl)
-    { return new SymbolListContext(sdl); }
+    static SymbolListContext *alloc(const SearchIndexList::const_iterator &start,
+                                    const SearchIndexList::const_iterator &end)
+    { return new SymbolListContext(start,end); }
 
     // TemplateListIntf
     virtual uint count() const;
@@ -1210,7 +1217,8 @@ class SymbolListContext : public RefCountedContext, public TemplateListIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    SymbolListContext(const SearchDefinitionList *sdl);
+    SymbolListContext(const SearchIndexList::const_iterator &start,
+                      const SearchIndexList::const_iterator &end);
    ~SymbolListContext();
     class Private;
     Private *p;
@@ -1221,8 +1229,9 @@ class SymbolListContext : public RefCountedContext, public TemplateListIntf
 class SymbolGroupContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static SymbolGroupContext *alloc(const SearchDefinitionList *sdl)
-    { return new SymbolGroupContext(sdl); }
+    static SymbolGroupContext *alloc(const SearchIndexList::const_iterator &start,
+                                     const SearchIndexList::const_iterator &end)
+    { return new SymbolGroupContext(start,end); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -1230,7 +1239,8 @@ class SymbolGroupContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    SymbolGroupContext(const SearchDefinitionList *sdl);
+    SymbolGroupContext(const SearchIndexList::const_iterator &start,
+                       const SearchIndexList::const_iterator &end);
    ~SymbolGroupContext();
     class Private;
     Private *p;
@@ -1241,7 +1251,7 @@ class SymbolGroupContext : public RefCountedContext, public TemplateStructIntf
 class SymbolGroupListContext : public RefCountedContext, public TemplateListIntf
 {
   public:
-    static SymbolGroupListContext *alloc(const SearchIndexList *sil)
+    static SymbolGroupListContext *alloc(const SearchIndexList &sil)
     { return new SymbolGroupListContext(sil); }
 
     // TemplateListIntf
@@ -1252,7 +1262,7 @@ class SymbolGroupListContext : public RefCountedContext, public TemplateListIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    SymbolGroupListContext(const SearchIndexList *sil);
+    SymbolGroupListContext(const SearchIndexList &sil);
    ~SymbolGroupListContext();
     class Private;
     Private *p;
@@ -1263,8 +1273,9 @@ class SymbolGroupListContext : public RefCountedContext, public TemplateListIntf
 class SymbolIndexContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static SymbolIndexContext *alloc(const SearchIndexList *sl,const QCString &name)
-    { return new SymbolIndexContext(sl,name); }
+    static SymbolIndexContext *alloc(const std::string &letter,
+                                     const SearchIndexList &sl,const QCString &name)
+    { return new SymbolIndexContext(letter,sl,name); }
 
     // TemplateStructIntf methods
     virtual TemplateVariant get(const char *name) const;
@@ -1272,7 +1283,7 @@ class SymbolIndexContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    SymbolIndexContext(const SearchIndexList *sl,const QCString &name);
+    SymbolIndexContext(const std::string &letter,const SearchIndexList &sl,const QCString &name);
    ~SymbolIndexContext();
     class Private;
     Private *p;
@@ -1283,7 +1294,7 @@ class SymbolIndexContext : public RefCountedContext, public TemplateStructIntf
 class SymbolIndicesContext : public RefCountedContext, public TemplateListIntf
 {
   public:
-    static SymbolIndicesContext *alloc(const SearchIndexInfo *info)
+    static SymbolIndicesContext *alloc(const SearchIndexInfo &info)
     { return new SymbolIndicesContext(info); }
 
     // TemplateListIntf
@@ -1294,7 +1305,7 @@ class SymbolIndicesContext : public RefCountedContext, public TemplateListIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    SymbolIndicesContext(const SearchIndexInfo *info);
+    SymbolIndicesContext(const SearchIndexInfo &info);
    ~SymbolIndicesContext();
     class Private;
     Private *p;
@@ -1305,7 +1316,7 @@ class SymbolIndicesContext : public RefCountedContext, public TemplateListIntf
 class SearchIndexContext : public RefCountedContext, public TemplateStructIntf
 {
   public:
-    static SearchIndexContext *alloc(const SearchIndexInfo *info)
+    static SearchIndexContext *alloc(const SearchIndexInfo &info)
     { return new SearchIndexContext(info); }
 
     // TemplateStructIntf methods
@@ -1314,7 +1325,7 @@ class SearchIndexContext : public RefCountedContext, public TemplateStructIntf
     virtual int release() { return RefCountedContext::release(); }
 
   private:
-    SearchIndexContext(const SearchIndexInfo *info);
+    SearchIndexContext(const SearchIndexInfo &info);
    ~SearchIndexContext();
     class Private;
     Private *p;
