@@ -1,6 +1,22 @@
-#include "md5.h"
+/******************************************************************************
+ *
+ * Copyright (C) 1997-2020 by Dimitri van Heesch.
+ *
+ * Permission to use, copy, modify, and distribute this software and its
+ * documentation under the terms of the GNU General Public License is hereby
+ * granted. No representations are made about the suitability of this software
+ * for any purpose. It is provided "as is" without express or implied warranty.
+ * See the GNU General Public License for more details.
+ *
+ * Documents produced by Doxygen are derivative works derived from the
+ * input used in their production; they are not affected by this license.
+ *
+ */
+
+#include <algorithm>
 
 #include "dirdef.h"
+#include "md5.h"
 #include "filename.h"
 #include "doxygen.h"
 #include "util.h"
@@ -15,11 +31,11 @@
 #include "docparser.h"
 #include "definitionimpl.h"
 #include "filedef.h"
-#include <algorithm>
+
 
 //----------------------------------------------------------------------
 
-class DirDefImpl : public DefinitionImpl, public DirDef
+class DirDefImpl : public DefinitionMixin<DirDef>
 {
   public:
     DirDefImpl(const char *path);
@@ -40,7 +56,7 @@ class DirDefImpl : public DefinitionImpl, public DirDef
     virtual int level() const { return m_level; }
     virtual DirDef *parent() const { return m_parent; }
     virtual int dirCount() const { return m_dirCount; }
-    virtual const QDict<UsedDir> *usedDirs() const { return m_usedDirs; }
+    virtual const UsedDirLinkedMap &usedDirs() const { return m_usedDirs; }
     virtual bool isParentOf(const DirDef *dir) const;
     virtual bool depGraphIsTrivial() const;
     virtual QCString shortTitle() const;
@@ -79,7 +95,7 @@ class DirDefImpl : public DefinitionImpl, public DirDef
     int m_dirCount;
     int m_level;
     DirDef *m_parent;
-    QDict<UsedDir> *m_usedDirs;
+    UsedDirLinkedMap m_usedDirs;
 };
 
 DirDef *createDirDef(const char *path)
@@ -93,7 +109,7 @@ DirDef *createDirDef(const char *path)
 
 static int g_dirCount=0;
 
-DirDefImpl::DirDefImpl(const char *path) : DefinitionImpl(path,1,1,path)
+DirDefImpl::DirDefImpl(const char *path) : DefinitionMixin(path,1,1,path)
 {
   bool fullPathNames = Config_getBool(FULL_PATH_NAMES);
   // get display name (stripping the paths mentioned in STRIP_FROM_PATH)
@@ -117,8 +133,6 @@ DirDefImpl::DirDefImpl(const char *path) : DefinitionImpl(path,1,1,path)
   }
 
   m_fileList   = new FileList;
-  m_usedDirs   = new QDict<UsedDir>(257);
-  m_usedDirs->setAutoDelete(TRUE);
   m_dirCount   = g_dirCount++;
   m_level=-1;
   m_parent=0;
@@ -127,7 +141,6 @@ DirDefImpl::DirDefImpl(const char *path) : DefinitionImpl(path,1,1,path)
 DirDefImpl::~DirDefImpl()
 {
   delete m_fileList;
-  delete m_usedDirs;
 }
 
 bool DirDefImpl::isLinkableInProject() const
@@ -641,10 +654,10 @@ void DirDefImpl::addUsesDependency(DirDef *dir,FileDef *srcFd,
 
   // levels match => add direct dependency
   bool added=FALSE;
-  UsedDir *usedDir = m_usedDirs->find(dir->getOutputFileBase());
+  UsedDir *usedDir = m_usedDirs.find(dir->getOutputFileBase());
   if (usedDir) // dir dependency already present
   {
-     FilePair *usedPair = usedDir->findFilePair(
+     const FilePair *usedPair = usedDir->findFilePair(
          srcFd->getOutputFileBase()+dstFd->getOutputFileBase());
      if (usedPair==0) // new file dependency
      {
@@ -660,9 +673,9 @@ void DirDefImpl::addUsesDependency(DirDef *dir,FileDef *srcFd,
   else // new directory dependency
   {
     //printf("  => new file\n");
-    usedDir = new UsedDir(dir,inherited);
-    usedDir->addFileDep(srcFd,dstFd);
-    m_usedDirs->insert(dir->getOutputFileBase(),usedDir);
+    auto newUsedDir = std::make_unique<UsedDir>(dir,inherited);
+    newUsedDir->addFileDep(srcFd,dstFd);
+    usedDir = m_usedDirs.add(dir->getOutputFileBase(),std::move(newUsedDir));
     added=TRUE;
   }
   if (added)
@@ -717,14 +730,14 @@ void DirDefImpl::computeDependencies()
       }
     }
   }
-  if (m_usedDirs)
+
+  std::sort(m_usedDirs.begin(),m_usedDirs.end(),
+            [](const auto &u1,const auto &u2)
+            { return qstricmp(u1->dir()->getOutputFileBase(),u2->dir()->getOutputFileBase())<0; });
+
+  for (const auto& usedDirectory : m_usedDirs)
   {
-    QDictIterator<UsedDir> udi(*m_usedDirs);
-    UsedDir *udir;
-    for (udi.toFirst();(udir=udi.current());++udi)
-    {
-      udir->sort();
-    }
+    usedDirectory->sort();
   }
 }
 
@@ -740,7 +753,7 @@ bool DirDefImpl::isParentOf(const DirDef *dir) const
 
 bool DirDefImpl::depGraphIsTrivial() const
 {
-  return m_usedDirs->count()==0;
+  return m_usedDirs.empty();
 }
 
 //----------------------------------------------------------------------
@@ -786,13 +799,14 @@ FilePair *UsedDir::findFilePair(const char *name)
 DirDef *DirDefImpl::createNewDir(const char *path)
 {
   ASSERT(path!=0);
-  DirDef *dir = Doxygen::directories->find(path);
+  DirDef *dir = Doxygen::dirLinkedMap->find(path);
   if (dir==0) // new dir
   {
+    dir = Doxygen::dirLinkedMap->add(path,
+            std::unique_ptr<DirDef>(
+              createDirDef(path)));
     //printf("Adding new dir %s\n",path);
-    dir = createDirDef(path);
     //printf("createNewDir %s short=%s\n",path,dir->shortName().data());
-    Doxygen::directories->append(path,dir);
   }
   return dir;
 }
@@ -926,14 +940,11 @@ void DirRelation::writeDocumentation(OutputList &ol)
 static void computeCommonDirPrefix()
 {
   QCString path;
-  DirDef *dir;
-  DirSDict::Iterator sdi(*Doxygen::directories);
-  if (Doxygen::directories->count()>0) // we have at least one dir
+  auto it = Doxygen::dirLinkedMap->begin();
+  if (!Doxygen::dirLinkedMap->empty()) // we have at least one dir
   {
     // start will full path of first dir
-    sdi.toFirst();
-    dir=sdi.current();
-    path=dir->name();
+    path=(*it)->name();
     int i=path.findRev('/',(int)path.length()-2);
     path=path.left(i+1);
     bool done=FALSE;
@@ -946,8 +957,8 @@ static void computeCommonDirPrefix()
       while (!done)
       {
         uint l = path.length();
-        uint count=0;
-        for (sdi.toFirst();(dir=sdi.current());++sdi)
+        size_t count=0;
+        for (const auto &dir : *Doxygen::dirLinkedMap)
         {
           QCString dirName = dir->name();
           if (dirName.length()>path.length())
@@ -985,7 +996,7 @@ static void computeCommonDirPrefix()
           }
           count++;
         }
-        if (count==Doxygen::directories->count())
+        if (count==Doxygen::dirLinkedMap->size())
           // path matches for all directories -> found the common prefix
         {
           done=TRUE;
@@ -993,7 +1004,7 @@ static void computeCommonDirPrefix()
       }
     }
   }
-  for (sdi.toFirst();(dir=sdi.current());++sdi)
+  for (const auto &dir : *Doxygen::dirLinkedMap)
   {
     QCString diskName = dir->name().right(dir->name().length()-path.length());
     dir->setDiskName(diskName);
@@ -1012,7 +1023,7 @@ void buildDirectories()
       if (fd->getReference().isEmpty())
       {
         DirDef *dir;
-        if ((dir=Doxygen::directories->find(fd->getPath()))==0) // new directory
+        if ((dir=Doxygen::dirLinkedMap->find(fd->getPath()))==0) // new directory
         {
           dir = DirDefImpl::mergeDirectoryInTree(fd->getPath());
         }
@@ -1025,45 +1036,48 @@ void buildDirectories()
     }
   }
 
-  //DirDef *root = new DirDef("root:");
   // compute relations between directories => introduce container dirs.
-  DirDef *dir;
-  DirSDict::Iterator sdi(*Doxygen::directories);
-  for (sdi.toFirst();(dir=sdi.current());++sdi)
+  for (const auto &dir : *Doxygen::dirLinkedMap)
   {
     QCString name = dir->name();
     int i=name.findRev('/',(int)name.length()-2);
     if (i>0)
     {
-      DirDef *parent = Doxygen::directories->find(name.left(i+1));
+      DirDef *parent = Doxygen::dirLinkedMap->find(name.left(i+1));
       //if (parent==0) parent=root;
       if (parent)
       {
-        parent->addSubDir(dir);
+        parent->addSubDir(dir.get());
         //printf("DirDefImpl::addSubdir(): Adding subdir\n%s to\n%s\n",
         //  dir->displayName().data(), parent->displayName().data());
       }
     }
   }
-  for (sdi.toFirst();(dir=sdi.current());++sdi)
+
+  // sort the directory contents
+  for (const auto &dir : *Doxygen::dirLinkedMap)
   {
     dir->sort();
   }
-  Doxygen::directories->sort();
+
+  // short the directories themselves
+  std::sort(Doxygen::dirLinkedMap->begin(),
+            Doxygen::dirLinkedMap->end(),
+            [](const auto &d1,const auto &d2)
+            { return qstricmp(d1->shortName(),d2->shortName()) < 0; });
+
   computeCommonDirPrefix();
 }
 
 void computeDirDependencies()
 {
-  DirDef *dir;
-  DirSDict::Iterator sdi(*Doxygen::directories);
   // compute nesting level for each directory
-  for (sdi.toFirst();(dir=sdi.current());++sdi)
+  for (const auto &dir : *Doxygen::dirLinkedMap)
   {
     dir->setLevel();
   }
   // compute uses dependencies between directories
-  for (sdi.toFirst();(dir=sdi.current());++sdi)
+  for (const auto &dir : *Doxygen::dirLinkedMap)
   {
     //printf("computeDependencies for %s: #dirs=%d\n",dir->name().data(),Doxygen::directories.count());
     dir->computeDependencies();
@@ -1073,9 +1087,7 @@ void computeDirDependencies()
 
 void generateDirDocs(OutputList &ol)
 {
-  DirDef *dir;
-  DirSDict::Iterator sdi(*Doxygen::directories);
-  for (sdi.toFirst();(dir=sdi.current());++sdi)
+  for (const auto &dir : *Doxygen::dirLinkedMap)
   {
     ol.pushGeneratorState();
     if (!dir->hasDocumentation())
@@ -1087,9 +1099,7 @@ void generateDirDocs(OutputList &ol)
   }
   if (Config_getBool(DIRECTORY_GRAPH))
   {
-    SDict<DirRelation>::Iterator rdi(Doxygen::dirRelations);
-    DirRelation *dr;
-    for (rdi.toFirst();(dr=rdi.current());++rdi)
+    for (const auto &dr : Doxygen::dirRelations)
     {
       dr->writeDocumentation(ol);
     }
@@ -1101,3 +1111,31 @@ bool compareDirDefs(const DirDef *item1, const DirDef *item2)
   return qstricmp(item1->shortName(),item2->shortName()) < 0;
 }
 
+// --- Cast functions
+
+DirDef *toDirDef(Definition *d)
+{
+  if (d==0) return 0;
+  if (d && typeid(*d)==typeid(DirDefImpl))
+  {
+    return static_cast<DirDef*>(d);
+  }
+  else
+  {
+    return 0;
+  }
+}
+
+const DirDef *toDirDef(const Definition *d)
+{
+  if (d==0) return 0;
+  if (d && typeid(*d)==typeid(DirDefImpl))
+  {
+    return static_cast<const DirDef*>(d);
+  }
+  else
+  {
+    return 0;
+  }
+}
+
