@@ -15,8 +15,12 @@
  *
  */
 
+#include <algorithm>
+#include <vector>
+
 #include <ctype.h>
 #include <qregexp.h>
+
 #include "groupdef.h"
 #include "classdef.h"
 #include "filedef.h"
@@ -45,7 +49,7 @@
 
 //---------------------------------------------------------------------------
 
-class GroupDefImpl : public DefinitionImpl, public GroupDef
+class GroupDefImpl : public DefinitionMixin<GroupDef>
 {
   public:
     GroupDefImpl(const char *fileName,int line,const char *name,const char *title,const char *refFileName=0);
@@ -54,7 +58,7 @@ class GroupDefImpl : public DefinitionImpl, public GroupDef
     virtual DefType definitionType() const { return TypeGroup; }
     virtual QCString getOutputFileBase() const;
     virtual QCString anchor() const { return QCString(); }
-    virtual QCString displayName(bool=TRUE) const { return hasGroupTitle() ? m_title : DefinitionImpl::name(); }
+    virtual QCString displayName(bool=TRUE) const { return hasGroupTitle() ? m_title : DefinitionMixin::name(); }
     virtual const char *groupTitle() const { return m_title; }
     virtual void setGroupTitle( const char *newtitle );
     virtual bool hasGroupTitle( ) const { return m_titleSet; }
@@ -62,7 +66,7 @@ class GroupDefImpl : public DefinitionImpl, public GroupDef
     virtual bool addClass(const ClassDef *def);
     virtual bool addNamespace(const NamespaceDef *def);
     virtual void addGroup(const GroupDef *def);
-    virtual void addPage(PageDef *def);
+    virtual void addPage(const PageDef *def);
     virtual void addExample(const PageDef *def);
     virtual void addDir(DirDef *dd);
     virtual bool insertMember(MemberDef *def,bool docOnly=FALSE);
@@ -72,7 +76,7 @@ class GroupDefImpl : public DefinitionImpl, public GroupDef
     virtual void writeMemberPages(OutputList &ol);
     virtual void writeQuickMemberLinks(OutputList &ol,const MemberDef *currentMd) const;
     virtual void writeTagFile(FTextStream &);
-    virtual int  numDocMembers() const;
+    virtual size_t numDocMembers() const;
     virtual bool isLinkableInProject() const;
     virtual bool isLinkable() const;
     virtual bool isASubGroup() const;
@@ -91,24 +95,23 @@ class GroupDefImpl : public DefinitionImpl, public GroupDef
     virtual Definition *getGroupScope() const { return m_groupScope; }
 
     virtual MemberList *getMemberList(MemberListType lt) const;
-    virtual const QList<MemberList> &getMemberLists() const { return m_memberLists; }
+    virtual const MemberLists &getMemberLists() const { return m_memberLists; }
 
     /* user defined member groups */
-    virtual MemberGroupSDict *getMemberGroupSDict() const { return m_memberGroupSDict; }
+    virtual const MemberGroupList &getMemberGroups() const { return m_memberGroups; }
 
     virtual FileList *      getFiles() const        { return m_fileList; }
-    virtual ClassSDict *    getClasses() const      { return m_classSDict; }
-    virtual NamespaceSDict * getNamespaces() const  { return m_namespaceSDict; }
-    virtual GroupList *     getSubGroups() const    { return m_groupList; }
-    virtual PageSDict *     getPages() const        { return m_pageDict; }
-    virtual const DirList & getDirs() const         { return m_dirList; }
-    virtual PageSDict *     getExamples() const     { return m_exampleDict; }
+    virtual const ClassLinkedRefMap &getClasses() const         { return m_classes; }
+    virtual const NamespaceLinkedRefMap &getNamespaces() const  { return m_namespaces; }
+    virtual const GroupList &getSubGroups() const               { return m_groups; }
+    virtual const PageLinkedRefMap &getPages() const            { return m_pages; }
+    virtual const DirList & getDirs() const                     { return m_dirList; }
+    virtual const PageLinkedRefMap &getExamples() const         { return m_examples; }
     virtual bool hasDetailedDescription() const;
     virtual void sortSubGroups();
 
   private:
     void addMemberListToGroup(MemberList *,bool (MemberDef::*)() const);
-    MemberList *createMemberList(MemberListType lt);
     void addMemberToList(MemberListType lt,MemberDef *md);
     void writeMemberDeclarations(OutputList &ol,MemberListType lt,const QCString &title);
     void writeMemberDocumentation(OutputList &ol,MemberListType lt,const QCString &title);
@@ -136,17 +139,17 @@ class GroupDefImpl : public DefinitionImpl, public GroupDef
     bool                 m_titleSet;            // true if title is not the same as the name
     QCString             m_fileName;            // base name of the generated file
     FileList *           m_fileList;            // list of files in the group
-    ClassSDict *         m_classSDict;          // list of classes in the group
-    NamespaceSDict *     m_namespaceSDict;      // list of namespaces in the group
-    GroupList *          m_groupList;           // list of sub groups.
-    PageSDict *          m_pageDict;            // list of pages in the group
-    PageSDict *          m_exampleDict;         // list of examples in the group
+    ClassLinkedRefMap    m_classes;             // list of classes in the group
+    NamespaceLinkedRefMap m_namespaces;         // list of namespaces in the group
+    GroupList            m_groups;              // list of sub groups.
+    PageLinkedRefMap     m_pages;               // list of pages in the group
+    PageLinkedRefMap     m_examples;            // list of examples in the group
     DirList              m_dirList;             // list of directories in the group
     MemberList *         m_allMemberList;
     MemberNameInfoLinkedMap m_allMemberNameInfoLinkedMap;
     Definition *         m_groupScope;
-    QList<MemberList>    m_memberLists;
-    MemberGroupSDict *   m_memberGroupSDict;
+    MemberLists          m_memberLists;
+    MemberGroupList      m_memberGroups;
     bool                 m_subGrouping;
 
 };
@@ -161,14 +164,9 @@ GroupDef *createGroupDef(const char *fileName,int line,const char *name,
 //---------------------------------------------------------------------------
 
 GroupDefImpl::GroupDefImpl(const char *df,int dl,const char *na,const char *t,
-                   const char *refFileName) : DefinitionImpl(df,dl,1,na)
+                   const char *refFileName) : DefinitionMixin(df,dl,1,na)
 {
   m_fileList = new FileList;
-  m_classSDict = new ClassSDict(17);
-  m_groupList = new GroupList;
-  m_namespaceSDict = new NamespaceSDict(17);
-  m_pageDict = new PageSDict(17);
-  m_exampleDict = new PageSDict(17);
   if (refFileName)
   {
     m_fileName=stripExtension(refFileName);
@@ -178,8 +176,6 @@ GroupDefImpl::GroupDefImpl(const char *df,int dl,const char *na,const char *t,
     m_fileName = convertNameToFile(QCString("group_")+na);
   }
   setGroupTitle( t );
-  m_memberGroupSDict = new MemberGroupSDict;
-  m_memberGroupSDict->setAutoDelete(TRUE);
 
   m_allMemberList = new MemberList(MemberListType_allMembersList);
 
@@ -191,13 +187,7 @@ GroupDefImpl::GroupDefImpl(const char *df,int dl,const char *na,const char *t,
 GroupDefImpl::~GroupDefImpl()
 {
   delete m_fileList;
-  delete m_classSDict;
-  delete m_groupList;
-  delete m_namespaceSDict;
-  delete m_pageDict;
-  delete m_exampleDict;
   delete m_allMemberList;
-  delete m_memberGroupSDict;
 }
 
 void GroupDefImpl::setGroupTitle( const char *t )
@@ -218,9 +208,7 @@ void GroupDefImpl::setGroupTitle( const char *t )
 
 void GroupDefImpl::distributeMemberGroupDocumentation()
 {
-  MemberGroupSDict::Iterator mgli(*m_memberGroupSDict);
-  MemberGroup *mg;
-  for (;(mg=mgli.current());++mgli)
+  for (const auto &mg : m_memberGroups)
   {
     mg->distributeMemberGroupDocumentation();
   }
@@ -228,17 +216,15 @@ void GroupDefImpl::distributeMemberGroupDocumentation()
 
 void GroupDefImpl::findSectionsInDocumentation()
 {
+  docFindSections(briefDescription(),this,docFile());
   docFindSections(documentation(),this,docFile());
-  MemberGroupSDict::Iterator mgli(*m_memberGroupSDict);
-  MemberGroup *mg;
-  for (;(mg=mgli.current());++mgli)
+
+  for (const auto &mg : m_memberGroups)
   {
     mg->findSectionsInDocumentation(this);
   }
 
-  QListIterator<MemberList> mli(m_memberLists);
-  MemberList *ml;
-  for (mli.toFirst();(ml=mli.current());++mli)
+  for (auto &ml : m_memberLists)
   {
     if (ml->listType()&MemberListType_declarationLists)
     {
@@ -260,47 +246,12 @@ void GroupDefImpl::addFile(const FileDef *def)
 
 bool GroupDefImpl::addClass(const ClassDef *cd)
 {
-  static bool sortBriefDocs = Config_getBool(SORT_BRIEF_DOCS);
   if (cd->isHidden()) return FALSE;
   updateLanguage(cd);
   QCString qn = cd->name();
-  if (m_classSDict->find(qn)==0)
+  if (m_classes.find(qn)==0)
   {
-    //printf("--- addClass %s sort=%d\n",qn.data(),sortBriefDocs);
-    if (sortBriefDocs)
-    {
-      m_classSDict->inSort(qn,cd);
-    }
-    else
-    {
-      int i=qn.findRev("::");
-      if (i==-1) i=qn.find('.');
-      bool found=FALSE;
-      //printf("i=%d\n",i);
-      if (i>0)
-      {
-        // add nested classes (e.g. A::B, A::C) after their parent (A) in
-        // order of insertion
-        QCString scope = qn.left(i);
-        int j=m_classSDict->findAt(scope);
-        if (j!=-1)
-        {
-          while (j<(int)m_classSDict->count() &&
-                        m_classSDict->at(j)->qualifiedName().left(i)==scope)
-          {
-            //printf("skipping over %s\n",classSDict->at(j)->qualifiedName().data());
-            j++;
-          }
-          //printf("Found scope at index %d\n",j);
-          m_classSDict->insertAt(j,qn,cd);
-          found=TRUE;
-        }
-      }
-      if (!found) // no insertion point found -> just append
-      {
-        m_classSDict->append(qn,cd);
-      }
-    }
+    m_classes.add(qn,cd);
     return TRUE;
   }
   return FALSE;
@@ -308,18 +259,14 @@ bool GroupDefImpl::addClass(const ClassDef *cd)
 
 bool GroupDefImpl::addNamespace(const NamespaceDef *def)
 {
-  static bool sortBriefDocs = Config_getBool(SORT_BRIEF_DOCS);
-  if (def->isHidden()) return FALSE;
-  updateLanguage(def);
-  if (m_namespaceSDict->find(def->name())==0)
+  if (def->isHidden()) return false;
+  if (m_namespaces.find(def->name())!=0)
   {
-    if (sortBriefDocs)
-      m_namespaceSDict->inSort(def->name(),def);
-    else
-      m_namespaceSDict->append(def->name(),def);
-    return TRUE;
+    updateLanguage(def);
+    m_namespaces.add(def->name(),def);
+    return true;
   }
-  return FALSE;
+  return false;
 }
 
 void GroupDefImpl::addDir(DirDef *def)
@@ -328,37 +275,33 @@ void GroupDefImpl::addDir(DirDef *def)
   m_dirList.push_back(def);
 }
 
-void GroupDefImpl::addPage(PageDef *def)
+void GroupDefImpl::addPage(const PageDef *def)
 {
   if (def->isHidden()) return;
   //printf("Making page %s part of a group\n",def->name.data());
-  m_pageDict->append(def->name(),def);
-  def->makePartOfGroup(this);
+  m_pages.add(def->name(),def);
+  const_cast<PageDef*>(def)->makePartOfGroup(this);
 }
 
 void GroupDefImpl::addExample(const PageDef *def)
 {
   if (def->isHidden()) return;
-  m_exampleDict->append(def->name(),def);
+  m_examples.add(def->name(),def);
 }
 
 
 void GroupDefImpl::addMembersToMemberGroup()
 {
-  QListIterator<MemberList> mli(m_memberLists);
-  MemberList *ml;
-  for (mli.toFirst();(ml=mli.current());++mli)
+  for (auto &ml : m_memberLists)
   {
     if (ml->listType()&MemberListType_declarationLists)
     {
-      ::addMembersToMemberGroup(ml,&m_memberGroupSDict,this);
+      ::addMembersToMemberGroup(ml.get(),&m_memberGroups,this);
     }
   }
 
   //printf("GroupDefImpl::addMembersToMemberGroup() memberGroupList=%d\n",memberGroupList->count());
-  MemberGroupSDict::Iterator mgli(*m_memberGroupSDict);
-  MemberGroup *mg;
-  for (;(mg=mgli.current());++mgli)
+  for (const auto &mg : m_memberGroups)
   {
     mg->setInGroup(TRUE);
   }
@@ -395,13 +338,14 @@ bool GroupDefImpl::insertMember(MemberDef *md,bool docOnly)
         sameScope // both are found in the same scope
        )
     {
-      if (srcMd->getGroupAlias()==0)
+      MemberDefMutable *mdm = toMemberDefMutable(md);
+      if (mdm && srcMd->getGroupAlias()==0)
       {
-        md->setGroupAlias(srcMd);
+        mdm->setGroupAlias(srcMd);
       }
-      else if (md!=srcMd->getGroupAlias())
+      else if (mdm && md!=srcMd->getGroupAlias())
       {
-        md->setGroupAlias(srcMd->getGroupAlias());
+        mdm->setGroupAlias(srcMd->getGroupAlias());
       }
       return FALSE; // member is the same as one that is already added
     }
@@ -597,16 +541,11 @@ bool GroupDefImpl::findGroup(const GroupDef *def) const
   {
     return TRUE;
   }
-  else if (m_groupList)
+  for (const auto &gd : m_groups)
   {
-    GroupListIterator it(*m_groupList);
-    GroupDef *gd;
-    for (;(gd=it.current());++it)
+    if (gd->findGroup(def))
     {
-      if (gd->findGroup(def))
-      {
-        return TRUE;
-      }
+      return TRUE;
     }
   }
   return FALSE;
@@ -618,52 +557,44 @@ void GroupDefImpl::addGroup(const GroupDef *def)
   //if (Config_getBool(SORT_MEMBER_DOCS))
   //  groupList->inSort(def);
   //else
-  m_groupList->append(def);
+  m_groups.push_back(def);
 }
 
 bool GroupDefImpl::isASubGroup() const
 {
-  GroupList *groups = partOfGroups();
-  return groups!=0 && groups->count()!=0;
+  return !partOfGroups().empty();
 }
 
 void GroupDefImpl::countMembers()
 {
-  QListIterator<MemberList> mli(m_memberLists);
-  MemberList *ml;
-  for (;(ml=mli.current());++mli)
+  for (auto &ml : m_memberLists)
   {
     ml->countDecMembers();
     ml->countDocMembers();
   }
-  if (m_memberGroupSDict)
+  for (const auto &mg : m_memberGroups)
   {
-    MemberGroupSDict::Iterator mgli(*m_memberGroupSDict);
-    MemberGroup *mg;
-    for (;(mg=mgli.current());++mgli)
-    {
-      mg->countDecMembers();
-      mg->countDocMembers();
-    }
+    mg->countDecMembers();
+    mg->countDocMembers();
   }
 }
 
-int GroupDefImpl::numDocMembers() const
+size_t GroupDefImpl::numDocMembers() const
 {
   return m_fileList->count()+
-         m_classSDict->count()+
-         m_namespaceSDict->count()+
-         m_groupList->count()+
+         m_classes.size()+
+         m_namespaces.size()+
+         m_groups.size()+
          m_allMemberList->count()+
-         m_pageDict->count()+
-         m_exampleDict->count();
+         m_pages.size()+
+         m_examples.size();
 }
 
 /*! Compute the HTML anchor names for all members in the group */
 void GroupDefImpl::computeAnchors()
 {
   //printf("GroupDefImpl::computeAnchors()\n");
-  setAnchors(m_allMemberList);
+  m_allMemberList->setAnchors();
 }
 
 void GroupDefImpl::writeTagFile(FTextStream &tagFile)
@@ -681,34 +612,24 @@ void GroupDefImpl::writeTagFile(FTextStream &tagFile)
     {
       case LayoutDocEntry::GroupClasses:
         {
-          if (m_classSDict)
+          for (const auto &cd : m_classes)
           {
-            SDict<ClassDef>::Iterator ci(*m_classSDict);
-            ClassDef *cd;
-            for (ci.toFirst();(cd=ci.current());++ci)
+            if (cd->isLinkableInProject())
             {
-              if (cd->isLinkableInProject())
-              {
-                tagFile << "    <class kind=\"" << cd->compoundTypeString()
-                        << "\">" << convertToXML(cd->name()) << "</class>" << endl;
-              }
+              tagFile << "    <class kind=\"" << cd->compoundTypeString()
+                      << "\">" << convertToXML(cd->name()) << "</class>" << endl;
             }
           }
         }
         break;
       case LayoutDocEntry::GroupNamespaces:
         {
-          if (m_namespaceSDict)
+          for (const auto &nd : m_namespaces)
           {
-            SDict<NamespaceDef>::Iterator ni(*m_namespaceSDict);
-            NamespaceDef *nd;
-            for (ni.toFirst();(nd=ni.current());++ni)
+            if (nd->isLinkableInProject())
             {
-              if (nd->isLinkableInProject())
-              {
-                tagFile << "    <namespace>" << convertToXML(nd->name())
-                        << "</namespace>" << endl;
-              }
+              tagFile << "    <namespace>" << convertToXML(nd->name())
+                      << "</namespace>" << endl;
             }
           }
         }
@@ -731,24 +652,19 @@ void GroupDefImpl::writeTagFile(FTextStream &tagFile)
         break;
       case LayoutDocEntry::GroupPageDocs:
         {
-          if (m_pageDict)
+          for (const auto &pd : m_pages)
           {
-            PageSDict::Iterator pdi(*m_pageDict);
-            PageDef *pd=0;
-            for (pdi.toFirst();(pd=pdi.current());++pdi)
+            QCString pageName = pd->getOutputFileBase();
+            if (pd->isLinkableInProject())
             {
-              QCString pageName = pd->getOutputFileBase();
-              if (pd->isLinkableInProject())
-              {
-                tagFile << "    <page>" << convertToXML(pageName) << "</page>" << endl;
-              }
+              tagFile << "    <page>" << convertToXML(pageName) << "</page>" << endl;
             }
           }
         }
         break;
       case LayoutDocEntry::GroupDirs:
         {
-          for(const auto dd : m_dirList)
+          for (const auto &dd : m_dirList)
           {
             if (dd->isLinkableInProject())
             {
@@ -759,16 +675,11 @@ void GroupDefImpl::writeTagFile(FTextStream &tagFile)
         break;
       case LayoutDocEntry::GroupNestedGroups:
         {
-          if (m_groupList)
+          for (const auto &gd : m_groups)
           {
-            QListIterator<GroupDef> it(*m_groupList);
-            GroupDef *gd;
-            for (;(gd=it.current());++it)
+            if (gd->isVisible())
             {
-              if (gd->isVisible())
-              {
-                tagFile << "    <subgroup>" << convertToXML(gd->name()) << "</subgroup>" << endl;
-              }
+              tagFile << "    <subgroup>" << convertToXML(gd->name()) << "</subgroup>" << endl;
             }
           }
         }
@@ -785,14 +696,9 @@ void GroupDefImpl::writeTagFile(FTextStream &tagFile)
         break;
       case LayoutDocEntry::MemberGroups:
         {
-          if (m_memberGroupSDict)
+          for (const auto &mg : m_memberGroups)
           {
-            MemberGroupSDict::Iterator mgli(*m_memberGroupSDict);
-            MemberGroup *mg;
-            for (;(mg=mgli.current());++mgli)
-            {
-              mg->writeTagFile(tagFile);
-            }
+            mg->writeTagFile(tagFile);
           }
         }
         break;
@@ -811,7 +717,7 @@ void GroupDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title
      )
   {
     ol.pushGeneratorState();
-    if (m_pageDict->count()!=(uint)numDocMembers()) // not only pages -> classical layout
+    if (m_pages.size()!=numDocMembers()) // not only pages -> classical layout
     {
       ol.pushGeneratorState();
         ol.disable(OutputGenerator::Html);
@@ -888,9 +794,7 @@ void GroupDefImpl::writeBriefDescription(OutputList &ol)
       ol.writeString(" \n");
       ol.enable(OutputGenerator::RTF);
 
-      if (Config_getBool(REPEAT_BRIEF) ||
-          !documentation().isEmpty()
-         )
+      if (hasDetailedDescription())
       {
         ol.disableAllBut(OutputGenerator::Html);
         ol.startTextLink(0,"details");
@@ -961,21 +865,16 @@ void GroupDefImpl::writeFiles(OutputList &ol,const QCString &title)
 void GroupDefImpl::writeNamespaces(OutputList &ol,const QCString &title)
 {
   // write list of namespaces
-  m_namespaceSDict->writeDeclaration(ol,title);
+  m_namespaces.writeDeclaration(ol,title);
 }
 
 void GroupDefImpl::writeNestedGroups(OutputList &ol,const QCString &title)
 {
   // write list of groups
   int count=0;
-  if (m_groupList->count()>0)
+  for (const auto &gd : m_groups)
   {
-    QListIterator<GroupDef> it(*m_groupList);
-    GroupDef *gd;
-    for (;(gd=it.current());++it)
-    {
-      if (gd->isVisible()) count++;
-    }
+    if (gd->isVisible()) count++;
   }
   if (count>0)
   {
@@ -983,13 +882,7 @@ void GroupDefImpl::writeNestedGroups(OutputList &ol,const QCString &title)
     ol.parseText(title);
     ol.endMemberHeader();
     ol.startMemberList();
-    if (Config_getBool(SORT_GROUP_NAMES))
-    {
-      m_groupList->sort();
-    }
-    QListIterator<GroupDef> it(*m_groupList);
-    GroupDef *gd;
-    for (;(gd=it.current());++it)
+    for (const auto &gd : m_groups)
     {
       if (gd->isVisible())
       {
@@ -1050,19 +943,17 @@ void GroupDefImpl::writeDirs(OutputList &ol,const QCString &title)
 void GroupDefImpl::writeClasses(OutputList &ol,const QCString &title)
 {
   // write list of classes
-  m_classSDict->writeDeclaration(ol,0,title,FALSE);
+  m_classes.writeDeclaration(ol,0,title,FALSE);
 }
 
 void GroupDefImpl::writeInlineClasses(OutputList &ol)
 {
-  m_classSDict->writeDocumentation(ol);
+  m_classes.writeDocumentation(ol);
 }
 
 void GroupDefImpl::writePageDocumentation(OutputList &ol)
 {
-  PageDef *pd=0;
-  PageSDict::Iterator pdi(*m_pageDict);
-  for (pdi.toFirst();(pd=pdi.current());++pdi)
+  for (const auto *pd : m_pages)
   {
     if (!pd->isReference())
     {
@@ -1085,16 +976,9 @@ void GroupDefImpl::writePageDocumentation(OutputList &ol)
 void GroupDefImpl::writeMemberGroups(OutputList &ol)
 {
   /* write user defined member groups */
-  if (m_memberGroupSDict)
+  for (const auto &mg : m_memberGroups)
   {
-    m_memberGroupSDict->sort();
-    /* write user defined member groups */
-    MemberGroupSDict::Iterator mgli(*m_memberGroupSDict);
-    MemberGroup *mg;
-    for (;(mg=mgli.current());++mgli)
-    {
-      mg->writeDeclarations(ol,0,0,0,this);
-    }
+    mg->writeDeclarations(ol,0,0,0,this);
   }
 }
 
@@ -1152,10 +1036,10 @@ void GroupDefImpl::writeSummaryLinks(OutputList &ol) const
   SrcLangExt lang = getLanguage();
   for (eli.toFirst();(lde=eli.current());++eli)
   {
-    if ((lde->kind()==LayoutDocEntry::GroupClasses && m_classSDict->declVisible()) ||
-        (lde->kind()==LayoutDocEntry::GroupNamespaces && m_namespaceSDict->declVisible()) ||
+    if ((lde->kind()==LayoutDocEntry::GroupClasses && m_classes.declVisible()) ||
+        (lde->kind()==LayoutDocEntry::GroupNamespaces && m_namespaces.declVisible()) ||
         (lde->kind()==LayoutDocEntry::GroupFiles && m_fileList->count()>0) ||
-        (lde->kind()==LayoutDocEntry::GroupNestedGroups && m_groupList->count()>0) ||
+        (lde->kind()==LayoutDocEntry::GroupNestedGroups && !m_groups.empty()) ||
         (lde->kind()==LayoutDocEntry::GroupDirs && !m_dirList.empty())
        )
     {
@@ -1373,9 +1257,7 @@ void GroupDefImpl::writeMemberPages(OutputList &ol)
   ol.pushGeneratorState();
   ol.disableAllBut(OutputGenerator::Html);
 
-  QListIterator<MemberList> mli(m_memberLists);
-  MemberList *ml;
-  for (mli.toFirst();(ml=mli.current());++mli)
+  for (const auto &ml : m_memberLists)
   {
     if (ml->listType()&MemberListType_documentationLists)
     {
@@ -1399,28 +1281,27 @@ void GroupDefImpl::writeQuickMemberLinks(OutputList &ol,const MemberDef *current
   {
     if (md->getGroupDef()==this && md->isLinkable() && !md->isEnumValue())
     {
-      ol.writeString("          <tr><td class=\"navtab\">");
       if (md->isLinkableInProject())
       {
         if (md==currentMd) // selected item => highlight
         {
-          ol.writeString("<a class=\"qindexHL\" ");
+          ol.writeString("          <tr><td class=\"navtabHL\">");
         }
         else
         {
-          ol.writeString("<a class=\"qindex\" ");
+          ol.writeString("          <tr><td class=\"navtab\">");
         }
+        ol.writeString("<a class=\"navtab\" ");
         ol.writeString("href=\"");
         if (createSubDirs) ol.writeString("../../");
         ol.writeString(md->getOutputFileBase()+Doxygen::htmlFileExtension+"#"+md->anchor());
         ol.writeString("\">");
         ol.writeString(convertToHtml(md->localName()));
         ol.writeString("</a>");
+        ol.writeString("</td></tr>\n");
       }
-      ol.writeString("</td></tr>\n");
     }
   }
-
   ol.writeString("        </table>\n");
   ol.writeString("      </div>\n");
 }
@@ -1433,12 +1314,13 @@ void addClassToGroups(const Entry *root,ClassDef *cd)
 {
   for (const Grouping &g : root->groups)
   {
-    GroupDef *gd=0;
-    if (!g.groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g.groupname)))
+    GroupDef *gd = Doxygen::groupLinkedMap->find(g.groupname);
+    if (gd && gd->addClass(cd))
     {
-      if (gd->addClass(cd))
+      ClassDefMutable *cdm = toClassDefMutable(cd);
+      if (cdm)
       {
-        cd->makePartOfGroup(gd);
+        cdm->makePartOfGroup(gd);
       }
       //printf("Compound %s: in group %s\n",cd->name().data(),gd->groupTitle());
     }
@@ -1450,11 +1332,15 @@ void addNamespaceToGroups(const Entry *root,NamespaceDef *nd)
   //printf("root->groups.size()=%d\n",root->groups.size());
   for (const Grouping &g : root->groups)
   {
-    GroupDef *gd=0;
+    GroupDef *gd = Doxygen::groupLinkedMap->find(g.groupname);
     //printf("group '%s'\n",s->data());
-    if (!g.groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g.groupname)))
+    if (gd && gd->addNamespace(nd))
     {
-      if (gd->addNamespace(nd)) nd->makePartOfGroup(gd);
+      NamespaceDefMutable *ndm = toNamespaceDefMutable(nd);
+      if (ndm)
+      {
+        ndm->makePartOfGroup(gd);
+      }
       //printf("Namespace %s: in group %s\n",nd->name().data(),s->data());
     }
   }
@@ -1465,9 +1351,9 @@ void addDirToGroups(const Entry *root,DirDef *dd)
   //printf("*** root->groups.size()=%d\n",root->groups.size());
   for (const Grouping &g : root->groups)
   {
-    GroupDef *gd=0;
+    GroupDef *gd = Doxygen::groupLinkedMap->find(g.groupname);
     //printf("group '%s'\n",g->groupname.data());
-    if (!g.groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g.groupname)))
+    if (gd)
     {
       gd->addDir(dd);
       dd->makePartOfGroup(gd);
@@ -1481,8 +1367,8 @@ void addGroupToGroups(const Entry *root,GroupDef *subGroup)
   //printf("addGroupToGroups for %s groups=%d\n",root->name.data(),root->groups.size());
   for (const Grouping &g : root->groups)
   {
-    GroupDef *gd=0;
-    if (!g.groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g.groupname)))
+    GroupDef *gd = Doxygen::groupLinkedMap->find(g.groupname);
+    if (gd)
     {
       if (gd==subGroup)
       {
@@ -1516,7 +1402,7 @@ void addMemberToGroups(const Entry *root,MemberDef *md)
   {
     GroupDef *gd=0;
     if (!g.groupname.isEmpty() &&
-        (gd=Doxygen::groupSDict->find(g.groupname)) &&
+        (gd=Doxygen::groupLinkedMap->find(g.groupname)) &&
         g.pri >= pri)
     {
       if (fgd && gd!=fgd && g.pri==pri)
@@ -1594,13 +1480,16 @@ void addMemberToGroups(const Entry *root,MemberDef *md)
       bool success = fgd->insertMember(md);
       if (success)
       {
-        //printf("insertMember successful\n");
-        md->setGroupDef(fgd,pri,root->fileName,root->startLine,
-            !root->doc.isEmpty());
-        ClassDef *cd = md->getClassDefOfAnonymousType();
-        if (cd)
+        MemberDefMutable *mdm = toMemberDefMutable(md);
+        if (mdm)
         {
-          cd->setGroupDefForAllMembers(fgd,pri,root->fileName,root->startLine,root->doc.length() != 0);
+          //printf("insertMember successful\n");
+          mdm->setGroupDef(fgd,pri,root->fileName,root->startLine,!root->doc.isEmpty());
+          ClassDefMutable *cdm = toClassDefMutable(mdm->getClassDefOfAnonymousType());
+          if (cdm)
+          {
+            cdm->setGroupDefForAllMembers(fgd,pri,root->fileName,root->startLine,root->doc.length() != 0);
+          }
         }
       }
     }
@@ -1612,8 +1501,8 @@ void addExampleToGroups(const Entry *root,PageDef *eg)
 {
   for (const Grouping &g : root->groups)
   {
-    GroupDef *gd=0;
-    if (!g.groupname.isEmpty() && (gd=Doxygen::groupSDict->find(g.groupname)))
+    GroupDef *gd = Doxygen::groupLinkedMap->find(g.groupname);
+    if (gd)
     {
       gd->addExample(eg);
       eg->makePartOfGroup(gd);
@@ -1639,15 +1528,11 @@ void GroupDefImpl::addListReferences()
              0
             );
   }
-  MemberGroupSDict::Iterator mgli(*m_memberGroupSDict);
-  MemberGroup *mg;
-  for (;(mg=mgli.current());++mgli)
+  for (const auto &mg : m_memberGroups)
   {
     mg->addListReferences(this);
   }
-  QListIterator<MemberList> mli(m_memberLists);
-  MemberList *ml;
-  for (mli.toFirst();(ml=mli.current());++mli)
+  for (auto &ml : m_memberLists)
   {
     if (ml->listType()&MemberListType_documentationLists)
     {
@@ -1656,59 +1541,117 @@ void GroupDefImpl::addListReferences()
   }
 }
 
-MemberList *GroupDefImpl::createMemberList(MemberListType lt)
-{
-  m_memberLists.setAutoDelete(TRUE);
-  QListIterator<MemberList> mli(m_memberLists);
-  MemberList *ml;
-  for (mli.toFirst();(ml=mli.current());++mli)
-  {
-    if (ml->listType()==lt)
-    {
-      return ml;
-    }
-  }
-  // not found, create a new member list
-  ml = new MemberList(lt);
-  m_memberLists.append(ml);
-  ml->setInGroup(TRUE);
-  return ml;
-}
-
 void GroupDefImpl::addMemberToList(MemberListType lt,MemberDef *md)
 {
   static bool sortBriefDocs = Config_getBool(SORT_BRIEF_DOCS);
   static bool sortMemberDocs = Config_getBool(SORT_MEMBER_DOCS);
-  MemberList *ml = createMemberList(lt);
+  const auto &ml = m_memberLists.get(lt);
+  ml->setInGroup(true);
   ml->setNeedsSorting(
       ((ml->listType()&MemberListType_declarationLists) && sortBriefDocs) ||
       ((ml->listType()&MemberListType_documentationLists) && sortMemberDocs));
   ml->append(md);
 }
 
+// performs a partial reordering to group elements together with the same scope
+template<class Vec>
+static void groupClassesWithSameScope(Vec &vec)
+{
+  bool done=false;
+  while (!done) // for each iteration
+  {
+    done=true;
+    for (size_t i=0; i<vec.size(); i++) // go through all items
+    {
+      std::string qni = vec[i]->name().str();
+      size_t posi = qni.rfind("::");
+      if (posi!=std::string::npos)
+      {
+        std::string scope = qni.substr(0,posi);
+        auto it = std::find_if( vec.begin(), vec.end(),
+            [&](typename Vec::Ptr &cd)
+            { return cd->name().str()==scope; });
+        if (it!=vec.end())
+        {
+          size_t idx = std::distance(vec.begin(),it);
+          if (i<idx) // parent scope located after child scope
+          {
+            // to avoid reordering elements with the same parent
+            // we skip to the last one with the same scope
+            size_t k = idx;
+            while (k<vec.size() && vec[k]->name().str().substr(0,posi)==scope)
+            {
+              idx = k;
+              k++;
+            }
+            // swap the items such that i is inserted after idx
+            for (size_t j=i; j<idx; j++)
+            {
+              std::swap(vec[j],vec[j+1]);
+            }
+            done=false;
+          }
+          else if (idx<i && vec[i-1]->name().str().substr(0,posi)!=scope)
+          {
+            // parent scope is found before the item, and the item
+            // has some other item with a different scope in front of it
+            // move idx to the end of range with the same scope
+            while (idx<i && vec[idx]->name().str().substr(0,posi)==scope)
+            {
+              idx++;
+            }
+            // swap the items such that i is just after idx
+            for (size_t j=idx; j<i; j++)
+            {
+              std::swap(vec[j],vec[j+1]);
+            }
+            done=false;
+          }
+        }
+      }
+    }
+  }
+}
+
 void GroupDefImpl::sortMemberLists()
 {
-  QListIterator<MemberList> mli(m_memberLists);
-  MemberList *ml;
-  for (;(ml=mli.current());++mli)
+  for (auto &ml : m_memberLists)
   {
     if (ml->needsSorting()) { ml->sort(); ml->setNeedsSorting(FALSE); }
   }
   if (Config_getBool(SORT_BRIEF_DOCS))
   {
     std::sort(m_dirList.begin(), m_dirList.end(), compareDirDefs);
+
+    auto classComp = [](const ClassLinkedRefMap::Ptr &c1,const ClassLinkedRefMap::Ptr &c2)
+    {
+      return Config_getBool(SORT_BY_SCOPE_NAME)     ?
+        qstricmp(c1->name(), c2->name())<0          :
+        qstricmp(c1->className(), c2->className())<0;
+    };
+    std::sort(m_classes.begin(), m_classes.end(), classComp);
+
+    auto namespaceComp = [](const NamespaceLinkedRefMap::Ptr &n1,const NamespaceLinkedRefMap::Ptr &n2)
+    {
+      return qstricmp(n1->name(),n2->name())<0;
+    };
+
+    std::sort(m_namespaces.begin(),m_namespaces.end(),namespaceComp);
+  }
+  else
+  {
+    groupClassesWithSameScope(m_classes);
+    groupClassesWithSameScope(m_namespaces);
   }
 }
 
 MemberList *GroupDefImpl::getMemberList(MemberListType lt) const
 {
-  QListIterator<MemberList> mli(m_memberLists);
-  MemberList *ml;
-  for (;(ml=mli.current());++mli)
+  for (auto &ml : m_memberLists)
   {
     if (ml->listType()==lt)
     {
-      return ml;
+      return ml.get();
     }
   }
   return 0;
@@ -1744,7 +1687,10 @@ void GroupDefImpl::removeMemberFromList(MemberListType lt,MemberDef *md)
 
 void GroupDefImpl::sortSubGroups()
 {
-  m_groupList->sort();
+  std::sort(m_groups.begin(),
+            m_groups.end(),
+            [](const auto &g1,const auto &g2)
+            { return qstrcmp(g1->groupTitle(),g2->groupTitle())<0; });
 }
 
 bool GroupDefImpl::isLinkableInProject() const
@@ -1771,6 +1717,37 @@ bool GroupDefImpl::hasDetailedDescription() const
 {
   static bool repeatBrief = Config_getBool(REPEAT_BRIEF);
   return ((!briefDescription().isEmpty() && repeatBrief) ||
-          !documentation().isEmpty());
+         !documentation().isEmpty() ||
+         !inbodyDocumentation().isEmpty()) &&
+         (m_pages.size()!=numDocMembers());
 }
 
+// --- Cast functions
+
+GroupDef *toGroupDef(Definition *d)
+{
+  if (d==0) return 0;
+  if (d && typeid(*d)==typeid(GroupDefImpl))
+  {
+    return static_cast<GroupDef*>(d);
+  }
+  else
+  {
+    return 0;
+  }
+}
+
+const GroupDef *toGroupDef(const Definition *d)
+{
+  if (d==0) return 0;
+  if (d && typeid(*d)==typeid(GroupDefImpl))
+  {
+    return static_cast<const GroupDef*>(d);
+  }
+  else
+  {
+    return 0;
+  }
+}
+
+
