@@ -74,15 +74,23 @@ static QCString getSubdir()
   return dir;
 }
 
-ManGenerator::ManGenerator() : OutputGenerator()
+ManGenerator::ManGenerator() : OutputGenerator(Config_getString(MAN_OUTPUT)+"/"+getSubdir())
 {
-  m_dir=Config_getString(MAN_OUTPUT) + "/" + getSubdir();
-  m_firstCol=TRUE;
-  m_paragraph=TRUE;
-  m_col=0;
-  m_upperCase=FALSE;
-  m_insideTabbing=FALSE;
-  m_inHeader=FALSE;
+}
+
+ManGenerator::ManGenerator(const ManGenerator &og) : OutputGenerator(og)
+{
+}
+
+ManGenerator &ManGenerator::operator=(const ManGenerator &og)
+{
+  OutputGenerator::operator=(og);
+  return *this;
+}
+
+std::unique_ptr<OutputGenerator> ManGenerator::clone() const
+{
+  return std::make_unique<ManGenerator>(*this);
 }
 
 ManGenerator::~ManGenerator()
@@ -147,7 +155,7 @@ static QCString buildFileName(const char *name)
   return fileName;
 }
 
-void ManGenerator::startFile(const char *,const char *manName,const char *)
+void ManGenerator::startFile(const char *,const char *manName,const char *,int)
 {
   startPlainFile( buildFileName( manName ) );
   m_firstCol=TRUE;
@@ -383,7 +391,7 @@ void ManGenerator::endItemListItem()
 {
 }
 
-void ManGenerator::startCodeFragment()
+void ManGenerator::startCodeFragment(const char *)
 {
   newParagraph();
   t << ".nf" << endl;
@@ -391,7 +399,7 @@ void ManGenerator::startCodeFragment()
   m_paragraph=FALSE;
 }
 
-void ManGenerator::endCodeFragment()
+void ManGenerator::endCodeFragment(const char *)
 {
   if (!m_firstCol) t << endl;
   t << ".fi" << endl;
@@ -428,7 +436,7 @@ void ManGenerator::startDoxyAnchor(const char *,const char *manName,
     //       name,baseName.data(),buildFileName(baseName).data());
 
     // - remove dangerous characters and append suffix, then add dir prefix
-    QCString fileName=m_dir+"/"+buildFileName( baseName );
+    QCString fileName=dir()+"/"+buildFileName( baseName );
     QFile linkfile( fileName );
     // - only create file if it doesn't exist already
     if ( !linkfile.open( IO_ReadOnly ) )
@@ -688,7 +696,7 @@ void ManGenerator::endParamList()
 {
 }
 
-void ManGenerator::writeDoc(DocNode *n,const Definition *ctx,const MemberDef *)
+void ManGenerator::writeDoc(DocNode *n,const Definition *ctx,const MemberDef *,int)
 {
   ManDocVisitor *visitor = new ManDocVisitor(t,*this,ctx?ctx->getDefFileExtension():QCString(""));
   n->accept(visitor);
