@@ -1264,13 +1264,9 @@ I think the hurdles are:
   inner_refid (unless I'm missing a method that would uniformly return
   the correct refid for all types).
 */
-static void writeInnerClasses(const ClassSDict *cl, struct Refid outer_refid)
+static void writeInnerClasses(const ClassLinkedRefMap &cl, struct Refid outer_refid)
 {
-  if (!cl) return;
-
-  ClassSDict::Iterator cli(*cl);
-  const ClassDef *cd;
-  for (cli.toFirst();(cd=cli.current());++cli)
+  for (const auto &cd : cl)
   {
     if (!cd->isHidden() && !cd->isAnonymous())
     {
@@ -1283,13 +1279,9 @@ static void writeInnerClasses(const ClassSDict *cl, struct Refid outer_refid)
   }
 }
 
-static void writeInnerPages(const PageSDict *pl, struct Refid outer_refid)
+static void writeInnerPages(const PageLinkedRefMap &pl, struct Refid outer_refid)
 {
-  if (!pl) return;
-
-  PageSDict::Iterator pli(*pl);
-  const PageDef *pd;
-  for (pli.toFirst();(pd=pli.current());++pli)
+  for (const auto &pd : pl)
   {
     struct Refid inner_refid = insertRefid(
       pd->getGroupDef() ? pd->getOutputFileBase()+"_"+pd->name() : pd->getOutputFileBase()
@@ -1298,24 +1290,18 @@ static void writeInnerPages(const PageSDict *pl, struct Refid outer_refid)
     bindIntParameter(contains_insert,":inner_rowid", inner_refid.rowid);
     bindIntParameter(contains_insert,":outer_rowid", outer_refid.rowid);
     step(contains_insert);
-
   }
 }
 
-static void writeInnerGroups(const GroupList *gl, struct Refid outer_refid)
+static void writeInnerGroups(const GroupList &gl, struct Refid outer_refid)
 {
-  if (gl)
+  for (const auto &sgd : gl)
   {
-    GroupListIterator gli(*gl);
-    const GroupDef *sgd;
-    for (gli.toFirst();(sgd=gli.current());++gli)
-    {
-      struct Refid inner_refid = insertRefid(sgd->getOutputFileBase());
+    struct Refid inner_refid = insertRefid(sgd->getOutputFileBase());
 
-      bindIntParameter(contains_insert,":inner_rowid", inner_refid.rowid);
-      bindIntParameter(contains_insert,":outer_rowid", outer_refid.rowid);
-      step(contains_insert);
-    }
+    bindIntParameter(contains_insert,":inner_rowid", inner_refid.rowid);
+    bindIntParameter(contains_insert,":outer_rowid", outer_refid.rowid);
+    step(contains_insert);
   }
 }
 
@@ -1348,22 +1334,17 @@ static void writeInnerDirs(const DirList &dl, struct Refid outer_refid)
   }
 }
 
-static void writeInnerNamespaces(const NamespaceSDict *nl, struct Refid outer_refid)
+static void writeInnerNamespaces(const NamespaceLinkedRefMap &nl, struct Refid outer_refid)
 {
-  if (nl)
+  for (const auto &nd : nl)
   {
-    NamespaceSDict::Iterator nli(*nl);
-    const NamespaceDef *nd;
-    for (nli.toFirst();(nd=nli.current());++nli)
+    if (!nd->isHidden() && !nd->isAnonymous())
     {
-      if (!nd->isHidden() && !nd->isAnonymous())
-      {
-        struct Refid inner_refid = insertRefid(nd->getOutputFileBase());
+      struct Refid inner_refid = insertRefid(nd->getOutputFileBase());
 
-        bindIntParameter(contains_insert,":inner_rowid",inner_refid.rowid);
-        bindIntParameter(contains_insert,":outer_rowid",outer_refid.rowid);
-        step(contains_insert);
-      }
+      bindIntParameter(contains_insert,":inner_rowid",inner_refid.rowid);
+      bindIntParameter(contains_insert,":outer_rowid",outer_refid.rowid);
+      step(contains_insert);
     }
   }
 }
@@ -1421,7 +1402,7 @@ QCString getSQLDocBlock(const Definition *scope,
     fileName,
     lineNr,
     const_cast<Definition*>(scope),
-    dynamic_cast<const MemberDef*>(def),
+    toMemberDef(def),
     doc,
     FALSE,
     FALSE,
@@ -1619,26 +1600,16 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
     // + source references
     // The cross-references in initializers only work when both the src and dst
     // are defined.
-    MemberSDict *mdict = md->getReferencesMembers();
-    if (mdict!=0)
+    auto refList = md->getReferencesMembers();
+    for (const auto &rmd : refList)
     {
-      MemberSDict::IteratorDict mdi(*mdict);
-      const MemberDef *rmd;
-      for (mdi.toFirst();(rmd=mdi.current());++mdi)
-      {
-        insertMemberReference(md,rmd, "inline");
-      }
+      insertMemberReference(md,rmd, "inline");
     }
     // + source referenced by
-    mdict = md->getReferencedByMembers();
-    if (mdict!=0)
+    auto refByList = md->getReferencedByMembers();
+    for (const auto &rmd : refByList)
     {
-      MemberSDict::IteratorDict mdi(*mdict);
-      const MemberDef *rmd;
-      for (mdi.toFirst();(rmd=mdi.current());++mdi)
-      {
-        insertMemberReference(rmd,md, "inline");
-      }
+      insertMemberReference(rmd,md, "inline");
     }
     return;
   }
@@ -1856,24 +1827,16 @@ static void generateSqlite3ForMember(const MemberDef *md, struct Refid scope_ref
   // + source references
   // The cross-references in initializers only work when both the src and dst
   // are defined.
-  MemberSDict *mdict = md->getReferencesMembers();
-  if (mdict!=0)
+  auto refList = md->getReferencesMembers();
+  for (const auto &refmd : refList)
   {
-    MemberSDict::IteratorDict mdi(*mdict);
-    for (mdi.toFirst();(rmd=mdi.current());++mdi)
-    {
-      insertMemberReference(md,rmd, "inline");
-    }
+    insertMemberReference(md,refmd, "inline");
   }
   // + source referenced by
-  mdict = md->getReferencedByMembers();
-  if (mdict!=0)
+  auto refByList = md->getReferencedByMembers();
+  for (const auto &refmd : refByList)
   {
-    MemberSDict::IteratorDict mdi(*mdict);
-    for (mdi.toFirst();(rmd=mdi.current());++mdi)
-    {
-      insertMemberReference(rmd,md, "inline");
-    }
+    insertMemberReference(refmd,md, "inline");
   }
 }
 
@@ -2012,65 +1975,48 @@ static void generateSqlite3ForClass(const ClassDef *cd)
   step(compounddef_insert);
 
   // + list of direct super classes
-  if (cd->baseClasses())
+  for (const auto &bcd : cd->baseClasses())
   {
-    BaseClassListIterator bcli(*cd->baseClasses());
-    const BaseClassDef *bcd;
-    for (bcli.toFirst();(bcd=bcli.current());++bcli)
-    {
-      struct Refid base_refid = insertRefid(bcd->classDef->getOutputFileBase());
-      struct Refid derived_refid = insertRefid(cd->getOutputFileBase());
-      bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
-      bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
-      bindIntParameter(compoundref_insert,":prot",bcd->prot);
-      bindIntParameter(compoundref_insert,":virt",bcd->virt);
-      step(compoundref_insert);
-    }
+    struct Refid base_refid = insertRefid(bcd.classDef->getOutputFileBase());
+    struct Refid derived_refid = insertRefid(cd->getOutputFileBase());
+    bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
+    bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
+    bindIntParameter(compoundref_insert,":prot",bcd.prot);
+    bindIntParameter(compoundref_insert,":virt",bcd.virt);
+    step(compoundref_insert);
   }
 
   // + list of direct sub classes
-  if (cd->subClasses())
+  for (const auto &bcd : cd->subClasses())
   {
-    BaseClassListIterator bcli(*cd->subClasses());
-    const BaseClassDef *bcd;
-    for (bcli.toFirst();(bcd=bcli.current());++bcli)
-    {
-      struct Refid derived_refid = insertRefid(bcd->classDef->getOutputFileBase());
-      struct Refid base_refid = insertRefid(cd->getOutputFileBase());
-      bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
-      bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
-      bindIntParameter(compoundref_insert,":prot",bcd->prot);
-      bindIntParameter(compoundref_insert,":virt",bcd->virt);
-      step(compoundref_insert);
-    }
+    struct Refid derived_refid = insertRefid(bcd.classDef->getOutputFileBase());
+    struct Refid base_refid = insertRefid(cd->getOutputFileBase());
+    bindIntParameter(compoundref_insert,":base_rowid", base_refid.rowid);
+    bindIntParameter(compoundref_insert,":derived_rowid", derived_refid.rowid);
+    bindIntParameter(compoundref_insert,":prot",bcd.prot);
+    bindIntParameter(compoundref_insert,":virt",bcd.virt);
+    step(compoundref_insert);
   }
 
   // + list of inner classes
-  writeInnerClasses(cd->getClassSDict(),refid);
+  writeInnerClasses(cd->getClasses(),refid);
 
   // + template argument list(s)
   writeTemplateList(cd);
 
   // + member groups
-  if (cd->getMemberGroupSDict())
+  for (const auto &mg : cd->getMemberGroups())
   {
-    MemberGroupSDict::Iterator mgli(*cd->getMemberGroupSDict());
-    MemberGroup *mg;
-    for (;(mg=mgli.current());++mgli)
-    {
-      generateSqlite3Section(cd,mg->members(),refid,"user-defined",mg->header(),
-          mg->documentation());
-    }
+    generateSqlite3Section(cd,mg->members(),refid,"user-defined",mg->header(),
+        mg->documentation());
   }
 
   // this is just a list of *local* members
-  QListIterator<MemberList> mli(cd->getMemberLists());
-  MemberList *ml;
-  for (mli.toFirst();(ml=mli.current());++mli)
+  for (const auto &ml : cd->getMemberLists())
   {
     if ((ml->listType()&MemberListType_detailedLists)==0)
     {
-      generateSqlite3Section(cd,ml,refid,"user-defined");
+      generateSqlite3Section(cd,ml.get(),refid,"user-defined");
     }
   }
 
@@ -2110,31 +2056,24 @@ static void generateSqlite3ForNamespace(const NamespaceDef *nd)
   step(compounddef_insert);
 
   // + contained class definitions
-  writeInnerClasses(nd->getClassSDict(),refid);
+  writeInnerClasses(nd->getClasses(),refid);
 
   // + contained namespace definitions
-  writeInnerNamespaces(nd->getNamespaceSDict(),refid);
+  writeInnerNamespaces(nd->getNamespaces(),refid);
 
   // + member groups
-  if (nd->getMemberGroupSDict())
+  for (const auto &mg : nd->getMemberGroups())
   {
-    MemberGroupSDict::Iterator mgli(*nd->getMemberGroupSDict());
-    MemberGroup *mg;
-    for (;(mg=mgli.current());++mgli)
-    {
-      generateSqlite3Section(nd,mg->members(),refid,"user-defined",mg->header(),
-          mg->documentation());
-    }
+    generateSqlite3Section(nd,mg->members(),refid,"user-defined",mg->header(),
+        mg->documentation());
   }
 
   // + normal members
-  QListIterator<MemberList> mli(nd->getMemberLists());
-  MemberList *ml;
-  for (mli.toFirst();(ml=mli.current());++mli)
+  for (const auto &ml : nd->getMemberLists())
   {
     if ((ml->listType()&MemberListType_declarationLists)!=0)
     {
-      generateSqlite3Section(nd,ml,refid,"user-defined");
+      generateSqlite3Section(nd,ml.get(),refid,"user-defined");
     }
   }
 }
@@ -2272,37 +2211,24 @@ static void generateSqlite3ForFile(const FileDef *fd)
   }
 
   // + contained class definitions
-  if (fd->getClassSDict())
-  {
-    writeInnerClasses(fd->getClassSDict(),refid);
-  }
+  writeInnerClasses(fd->getClasses(),refid);
 
   // + contained namespace definitions
-  if (fd->getNamespaceSDict())
-  {
-    writeInnerNamespaces(fd->getNamespaceSDict(),refid);
-  }
+  writeInnerNamespaces(fd->getNamespaces(),refid);
 
   // + member groups
-  if (fd->getMemberGroupSDict())
+  for (const auto &mg : fd->getMemberGroups())
   {
-    MemberGroupSDict::Iterator mgli(*fd->getMemberGroupSDict());
-    MemberGroup *mg;
-    for (;(mg=mgli.current());++mgli)
-    {
-      generateSqlite3Section(fd,mg->members(),refid,"user-defined",mg->header(),
+    generateSqlite3Section(fd,mg->members(),refid,"user-defined",mg->header(),
           mg->documentation());
-    }
   }
 
   // + normal members
-  QListIterator<MemberList> mli(fd->getMemberLists());
-  MemberList *ml;
-  for (mli.toFirst();(ml=mli.current());++mli)
+  for (const auto &ml : fd->getMemberLists())
   {
     if ((ml->listType()&MemberListType_declarationLists)!=0)
     {
-      generateSqlite3Section(fd,ml,refid,"user-defined");
+      generateSqlite3Section(fd,ml.get(),refid,"user-defined");
     }
   }
 }
@@ -2358,25 +2284,18 @@ static void generateSqlite3ForGroup(const GroupDef *gd)
   writeInnerGroups(gd->getSubGroups(),refid);
 
   // + member groups
-  if (gd->getMemberGroupSDict())
+  for (const auto &mg : gd->getMemberGroups())
   {
-    MemberGroupSDict::Iterator mgli(*gd->getMemberGroupSDict());
-    MemberGroup *mg;
-    for (;(mg=mgli.current());++mgli)
-    {
-      generateSqlite3Section(gd,mg->members(),refid,"user-defined",mg->header(),
-          mg->documentation());
-    }
+    generateSqlite3Section(gd,mg->members(),refid,"user-defined",mg->header(),
+        mg->documentation());
   }
 
   // + members
-  QListIterator<MemberList> mli(gd->getMemberLists());
-  MemberList *ml;
-  for (mli.toFirst();(ml=mli.current());++mli)
+  for (const auto &ml : gd->getMemberLists())
   {
     if ((ml->listType()&MemberListType_declarationLists)!=0)
     {
-      generateSqlite3Section(gd,ml,refid,"user-defined");
+      generateSqlite3Section(gd,ml.get(),refid,"user-defined");
     }
   }
 }
@@ -2456,7 +2375,7 @@ static void generateSqlite3ForPage(const PageDef *pd,bool isExample)
   bindTextParameter(compounddef_insert,":name",pd->name());
 
   QCString title;
-  if (pd==Doxygen::mainPage) // main page is special
+  if (pd==Doxygen::mainPage.get()) // main page is special
   {
     if (mainPageHasTitle())
     {
@@ -2583,21 +2502,17 @@ void generateSqlite3()
   recordMetadata();
 
   // + classes
-  ClassSDict::Iterator cli(*Doxygen::classSDict);
-  const ClassDef *cd;
-  for (cli.toFirst();(cd=cli.current());++cli)
+  for (const auto &cd : *Doxygen::classLinkedMap)
   {
     msg("Generating Sqlite3 output for class %s\n",cd->name().data());
-    generateSqlite3ForClass(cd);
+    generateSqlite3ForClass(cd.get());
   }
 
   // + namespaces
-  NamespaceSDict::Iterator nli(*Doxygen::namespaceSDict);
-  const NamespaceDef *nd;
-  for (nli.toFirst();(nd=nli.current());++nli)
+  for (const auto &nd : *Doxygen::namespaceLinkedMap)
   {
     msg("Generating Sqlite3 output for namespace %s\n",nd->name().data());
-    generateSqlite3ForNamespace(nd);
+    generateSqlite3ForNamespace(nd.get());
   }
 
   // + files
@@ -2611,52 +2526,38 @@ void generateSqlite3()
   }
 
   // + groups
-  GroupSDict::Iterator gli(*Doxygen::groupSDict);
-  const GroupDef *gd;
-  for (;(gd=gli.current());++gli)
+  for (const auto &gd : *Doxygen::groupLinkedMap)
   {
     msg("Generating Sqlite3 output for group %s\n",gd->name().data());
-    generateSqlite3ForGroup(gd);
+    generateSqlite3ForGroup(gd.get());
   }
 
   // + page
+  for (const auto &pd : *Doxygen::pageLinkedMap)
   {
-    PageSDict::Iterator pdi(*Doxygen::pageSDict);
-    const PageDef *pd=0;
-    for (pdi.toFirst();(pd=pdi.current());++pdi)
-    {
-      msg("Generating Sqlite3 output for page %s\n",pd->name().data());
-      generateSqlite3ForPage(pd,FALSE);
-    }
+    msg("Generating Sqlite3 output for page %s\n",pd->name().data());
+    generateSqlite3ForPage(pd.get(),FALSE);
   }
 
   // + dirs
+  for (const auto &dd : *Doxygen::dirLinkedMap)
   {
-    const DirDef *dir;
-    DirSDict::Iterator sdi(*Doxygen::directories);
-    for (sdi.toFirst();(dir=sdi.current());++sdi)
-    {
-      msg("Generating Sqlite3 output for dir %s\n",dir->name().data());
-      generateSqlite3ForDir(dir);
-    }
+    msg("Generating Sqlite3 output for dir %s\n",dd->name().data());
+    generateSqlite3ForDir(dd.get());
   }
 
   // + examples
+  for (const auto &pd : *Doxygen::exampleLinkedMap)
   {
-    PageSDict::Iterator pdi(*Doxygen::exampleSDict);
-    const PageDef *pd=0;
-    for (pdi.toFirst();(pd=pdi.current());++pdi)
-    {
-      msg("Generating Sqlite3 output for example %s\n",pd->name().data());
-      generateSqlite3ForPage(pd,TRUE);
-    }
+    msg("Generating Sqlite3 output for example %s\n",pd->name().data());
+    generateSqlite3ForPage(pd.get(),TRUE);
   }
 
   // + main page
   if (Doxygen::mainPage)
   {
     msg("Generating Sqlite3 output for the main page\n");
-    generateSqlite3ForPage(Doxygen::mainPage,FALSE);
+    generateSqlite3ForPage(Doxygen::mainPage.get(),FALSE);
   }
 
   // TODO: copied from initializeSchema; not certain if we should say/do more
