@@ -23,6 +23,8 @@
  */
 
 #include <memory>
+#include <unordered_map>
+#include <algorithm>
 
 #include <qlist.h>
 #include <ctype.h>
@@ -32,6 +34,7 @@
 #include "classdef.h"
 #include "arguments.h"
 #include "containers.h"
+#include "namespacedef.h"
 
 //--------------------------------------------------------------------
 
@@ -45,12 +48,8 @@ class OutputList;
 class OutputDocInterface;
 class MemberDef;
 class ExampleSDict;
-class ClassSDict;
-class BaseClassList;
 class GroupDef;
 class NamespaceSDict;
-class ClassList;
-class MemberGroupSDict;
 struct TagInfo;
 class PageDef;
 class SectionInfo;
@@ -93,32 +92,6 @@ class TextGeneratorOLImpl : public TextGeneratorIntf
 
 //--------------------------------------------------------------------
 
-/** @brief maps a unicode character code to a list of T::ElementType's
- */
-template<class T>
-class LetterToIndexMap : public SIntDict<T>
-{
-  public:
-    LetterToIndexMap() { SIntDict<T>::setAutoDelete(TRUE); }
-    void append(uint letter,typename T::ElementType *elem)
-    {
-      T *l = SIntDict<T>::find((int)letter);
-      if (l==0)
-      {
-        l = new T(letter);
-        SIntDict<T>::inSort((int)letter,l);
-      }
-      l->append(elem);
-    }
-  private:
-    int compareValues(const T *l1, const T *l2) const
-    {
-      return (int)l1->letter()-(int)l2->letter();
-    }
-};
-
-//--------------------------------------------------------------------
-
 QCString langToString(SrcLangExt lang);
 QCString getLanguageSpecificSeparator(SrcLangExt lang,bool classScope=FALSE);
 
@@ -135,8 +108,6 @@ void linkifyText(const TextGeneratorIntf &ol,
                  int indentLevel=0
                 );
 
-void setAnchors(MemberList *ml);
-
 QCString fileToString(const char *name,bool filter=FALSE,bool isSourceCode=FALSE);
 
 QCString dateToString(bool);
@@ -151,8 +122,7 @@ bool getDefs(const QCString &scopeName,
                     const GroupDef *&gd,
                     bool forceEmptyScope=FALSE,
                     const FileDef *currentFile=0,
-                    bool checkCV=FALSE,
-                    const char *forceTagFile=0
+                    bool checkCV=FALSE
                    );
 
 QCString getFileFilter(const char* name,bool isSourceCode);
@@ -204,17 +174,16 @@ QCString selectBlock(const QCString& s,const QCString &name,bool which);
 QCString resolveDefines(const char *n);
 
 ClassDef *getClass(const char *key);
-
-const ClassDef *getResolvedClass(const Definition *scope,
-                           const FileDef *fileScope,
-                           const char *key,
-                           const MemberDef **pTypeDef=0,
-                           QCString *pTemplSpec=0,
-                           bool mayBeUnlinkable=FALSE,
-                           bool mayBeHidden=FALSE,
-                           QCString *pResolvedType=0);
+inline ClassDefMutable *getClassMutable(const char *key)
+{
+  return toClassDefMutable(getClass(key));
+}
 
 NamespaceDef *getResolvedNamespace(const char *key);
+inline NamespaceDefMutable *getResolvedNamespaceMutable(const char *key)
+{
+  return toNamespaceDefMutable(getResolvedNamespace(key));
+}
 
 FileDef *findFileDef(const FileNameLinkedMap *fnMap,const char *n,
                 bool &ambig);
@@ -232,11 +201,11 @@ QCString removeRedundantWhiteSpace(const QCString &s);
 
 QCString argListToString(const ArgumentList &al,bool useCanonicalType=FALSE,bool showDefVals=TRUE);
 
-QCString tempArgListToString(const ArgumentList &al,SrcLangExt lang);
+QCString tempArgListToString(const ArgumentList &al,SrcLangExt lang,bool includeDefaults=true);
 
 QCString generateMarker(int id);
 
-void writeExample(OutputList &ol,ExampleSDict *el);
+void writeExamples(OutputList &ol,const ExampleList &el);
 
 QCString stripAnonymousNamespaceScope(const QCString &s);
 
@@ -257,11 +226,10 @@ QCString removeAnonymousScopes(const QCString &s);
 
 QCString replaceAnonymousScopes(const QCString &s,const char *replacement=0);
 
-void initClassHierarchy(ClassSDict *cl);
-
-bool hasVisibleRoot(const BaseClassList *bcl);
+bool hasVisibleRoot(const BaseClassList &bcl);
 bool classHasVisibleChildren(const ClassDef *cd);
-bool namespaceHasVisibleChild(const NamespaceDef *nd,bool includeClasses,bool filterClasses,ClassDef::CompoundType ct);
+bool namespaceHasNestedNamespace(const NamespaceDef *nd);
+bool namespaceHasNestedClass(const NamespaceDef *nd,bool filterClasses,ClassDef::CompoundType ct);
 bool classVisibleInIndex(const ClassDef *cd);
 
 int minClassDistance(const ClassDef *cd,const ClassDef *bcd,int level=0);
@@ -295,7 +263,7 @@ QCString convertToPSString(const char *s);
 QCString getOverloadDocs();
 
 void addMembersToMemberGroup(/* in,out */ MemberList *ml,
-                             /* in,out */ MemberGroupSDict **ppMemberGroupSDict,
+                             /* in,out */ MemberGroupList *pMemberGroups,
                              /* in */     const Definition *context);
 
 int extractClassNameFromType(const QCString &type,int &pos,
@@ -311,8 +279,6 @@ QCString substituteTemplateArgumentsInString(
        const ArgumentList &formalArgs,
        const std::unique_ptr<ArgumentList> &actualArgs);
 
-//QList<ArgumentList> *copyArgumentLists(const QList<ArgumentList> *srcLists);
-
 QCString stripTemplateSpecifiersFromScope(const QCString &fullName,
                                           bool parentOnly=TRUE,
                                           QCString *lastScopeStripped=0);
@@ -337,7 +303,9 @@ void addRefItem(const RefItemVector &sli,
 PageDef *addRelatedPage(const char *name,
                         const QCString &ptitle,
                         const QCString &doc,
-                        const char *fileName,int startLine,
+                        const char *fileName,
+                        int docLine,
+                        int startLine,
                         const RefItemVector &sli = RefItemVector(),
                         GroupDef *gd=0,
                         const TagInfo *tagInfo=0,
@@ -351,10 +319,11 @@ QCString unescapeCharsInString(const char *s);
 void addGroupListToTitle(OutputList &ol,const Definition *d);
 
 void filterLatexString(FTextStream &t,const char *str,
-                       bool insideTabbing=FALSE,
-                       bool insidePre=FALSE,
-                       bool insideItem=FALSE,
-                       bool keepSpaces=FALSE);
+                       bool insideTabbing,
+                       bool insidePre,
+                       bool insideItem,
+                       bool insideTable,
+                       bool keepSpaces);
 
 QCString latexEscapeLabelName(const char *s);
 QCString latexEscapeIndexChars(const char *s);
@@ -375,11 +344,6 @@ QCString stripExtension(const char *fName);
 
 void replaceNamespaceAliases(QCString &scope,int i);
 
-int isAccessibleFrom(const Definition *scope,const FileDef *fileScope,const Definition *item);
-
-int isAccessibleFromWithExpScope(const Definition *scope,const FileDef *fileScope,const Definition *item,
-                     const QCString &explicitScopePart);
-
 int computeQualifiedIndex(const QCString &name);
 
 void addDirPrefix(QCString &fileName);
@@ -402,17 +366,8 @@ QCString getFileNameExtension(QCString fn);
 void initDefaultExtensionMapping();
 void addCodeOnlyMappings();
 
-MemberDef *getMemberFromSymbol(const Definition *scope,const FileDef *fileScope,
-                                const char *n);
 bool checkIfTypedef(const Definition *scope,const FileDef *fileScope,const char *n);
 
-const ClassDef *newResolveTypedef(const FileDef *fileScope,
-                                  const MemberDef *md,
-                                  const MemberDef **pMemType=0,
-                                  QCString *pTemplSpec=0,
-                                  QCString *pResolvedType=0,
-                                  const std::unique_ptr<ArgumentList> &actTemplParams=std::unique_ptr<ArgumentList>());
-
 QCString parseCommentAsText(const Definition *scope,const MemberDef *member,const QCString &doc,const QCString &fileName,int lineNr);
 
 QCString transcodeCharacterStringToUTF8(const QCString &input);
@@ -423,8 +378,6 @@ QCString extractAliasArgs(const QCString &args,int pos);
 
 int countAliasArguments(const QCString argList);
 
-//QCString replaceAliasArguments(const QCString &aliasValue,const QCString &argList);
-
 QCString resolveAliasCmd(const QCString aliasCmd);
 QCString expandAlias(const QCString &aliasName,const QCString &aliasValue);
 
@@ -474,12 +427,19 @@ bool protectionLevelVisible(Protection prot);
 QCString stripIndentation(const QCString &s);
 void stripIndentation(QCString &doc,const int indentationLevel);
 
-QCString getDotImageExtension(void);
+QCString getDotImageExtension();
 
 bool fileVisibleInIndex(const FileDef *fd,bool &genSourceFile);
 
-void addDocCrossReference(MemberDef *src,MemberDef *dst);
-
+const int MAX_UTF8_CHAR_LEN = 4;
+const int MAX_UTF8_CHAR_SIZE = MAX_UTF8_CHAR_LEN+1; // include 0 terminator
+enum class CaseModifier
+{
+  None,
+  ToUpper,
+  ToLower
+};
+int getUtf8Char(const char *input,char ids[MAX_UTF8_CHAR_SIZE],CaseModifier modifier=CaseModifier::None);
 uint getUtf8Code( const QCString& s, int idx );
 uint getUtf8CodeToLower( const QCString& s, int idx );
 uint getUtf8CodeToUpper( const QCString& s, int idx );
@@ -498,8 +458,7 @@ bool openOutputFile(const char *outFile,QFile &f);
 void writeExtraLatexPackages(FTextStream &t);
 void writeLatexSpecialFormulaChars(FTextStream &t);
 
-int usedTableLevels();
-void incUsedTableLevels();
-void decUsedTableLevels();
+bool recognizeFixedForm(const char* contents, FortranFormat format);
+FortranFormat convertFileNameFortranParserCode(QCString fn);
 
 #endif
