@@ -24,10 +24,10 @@
 #include <stdlib.h>
 #include <assert.h>
 #include <string.h>
+#include <map>
 #include <qcstring.h>
 #include <qfileinfo.h>
 #include <qcstringlist.h>
-#include <qmap.h>
 
 /* --------------------------------------------------------------- */
 
@@ -55,6 +55,7 @@
 #include "filename.h"
 #include "membergroup.h"
 #include "memberdef.h"
+#include "membername.h"
 #include "plantuml.h"
 #include "vhdljjparser.h"
 #include "VhdlParser.h"
@@ -63,14 +64,9 @@
 //#define DEBUGFLOW
 #define theTranslator_vhdlType theTranslator->trVhdlType
 
-static QDict<QCString> g_vhdlKeyDict0(17,FALSE);
-static QDict<QCString> g_vhdlKeyDict1(17,FALSE);
-static QDict<QCString> g_vhdlKeyDict2(17,FALSE);
-static QDict<QCString> g_vhdlKeyDict3(17,FALSE);
-
 static void initUCF(Entry* root,const char* type,QCString &  qcs,int line,QCString & fileName,QCString & brief);
 static void writeUCFLink(const MemberDef* mdef,OutputList &ol);
-static void addInstance(ClassDef* entity, ClassDef* arch, ClassDef *inst,
+static void addInstance(ClassDefMutable* entity, ClassDefMutable* arch, ClassDefMutable *inst,
                         const std::shared_ptr<Entry> &cur);
 
 //---------- create svg -------------------------------------------------------------
@@ -178,10 +174,8 @@ static void createSVG()
 // Brief descriptions for entities are shown too.
 void VhdlDocGen::writeOverview()
 {
-  ClassSDict::Iterator cli(*Doxygen::classSDict);
-  ClassDef *cd;
   bool found=FALSE;
-  for ( ; (cd=cli.current()) ; ++cli )
+  for (const auto &cd : *Doxygen::classLinkedMap)
   {
     if ((VhdlDocGen::VhdlClasses)cd->protection()==VhdlDocGen::ENTITYCLASS )
     {
@@ -205,14 +199,14 @@ void VhdlDocGen::writeOverview()
 
   startDot(t);
 
-  for (cli.toFirst() ; (cd=cli.current()) ; ++cli )
+  for (const auto &cd : *Doxygen::classLinkedMap)
   {
     if ((VhdlDocGen::VhdlClasses)cd->protection()!=VhdlDocGen::ENTITYCLASS )
     {
       continue;
     }
 
-    QList<MemberDef>* port= getPorts(cd);
+    QList<MemberDef>* port= getPorts(cd.get());
     if (port==0)
     {
       continue;
@@ -225,30 +219,23 @@ void VhdlDocGen::writeOverview()
     }
 
     startTable(t,cd->name());
-    writeClassToDot(t,cd);
+    writeClassToDot(t,cd.get());
     writeTable(port,t);
     endTable(t);
 
-   // writeVhdlPortToolTip(t,port,cd);
-    writeVhdlEntityToolTip(t,cd);
+    writeVhdlEntityToolTip(t,cd.get());
     delete port;
 
-    BaseClassList *bl=cd->baseClasses();
-    if (bl)
+    for (const auto &bcd : cd->baseClasses())
     {
-      BaseClassListIterator bcli(*bl);
-      BaseClassDef *bcd;
-      for ( ; (bcd=bcli.current()) ; ++bcli )
-      {
-        ClassDef *bClass=bcd->classDef;
-        QCString dotn=cd->name()+":";
-        dotn+=cd->name();
-        QCString csc=bClass->name()+":";
-        csc+=bClass->name();
-        //  fprintf(stderr,"\n <%s| %s>",dotn.data(),csc.data());
-        writeVhdlDotLink(t,dotn,csc,0);
-      }
-    }// if bl
+      ClassDef *bClass=bcd.classDef;
+      QCString dotn=cd->name()+":";
+      dotn+=cd->name();
+      QCString csc=bClass->name()+":";
+      csc+=bClass->name();
+      //  fprintf(stderr,"\n <%s| %s>",dotn.data(),csc.data());
+      writeVhdlDotLink(t,dotn,csc,0);
+    }
   }// for
 
   endDot(t);
@@ -567,11 +554,8 @@ VhdlDocGen::~VhdlDocGen()
 {
 }
 
-void VhdlDocGen::init()
-{
-
  // vhdl keywords included VHDL 2008
-const char* g_vhdlKeyWordMap0[] =
+static const std::set< std::string > g_vhdlKeyWordSet0 =
 {
   "abs","access","after","alias","all","and","architecture","array","assert","assume","assume_guarantee","attribute",
   "begin","block","body","buffer","bus",
@@ -592,100 +576,57 @@ const char* g_vhdlKeyWordMap0[] =
   "unaffected","units","until","use",
   "variable","vmode","vprop","vunit",
   "wait","when","while","with",
-  "xor","xnor",
-  0
+  "xor","xnor"
 };
 
 
 // type
-const char* g_vhdlKeyWordMap1[] =
+static const std::set< std::string> g_vhdlKeyWordSet1 =
 {
   "natural","unsigned","signed","string","boolean", "bit","bit_vector","character",
   "std_ulogic","std_ulogic_vector","std_logic","std_logic_vector","integer",
-  "real","float","ufixed","sfixed","time","positive",0
+  "real","float","ufixed","sfixed","time","positive"
 };
 
 // logic
-const char* g_vhdlKeyWordMap2[] =
+static const std::set< std::string > g_vhdlKeyWordSet2 =
 {
-  "abs","and","or","not","mod", "xor","rem","xnor","ror","rol","sla",
-  "sll",0
+  "abs","and","or","not","mod","xor","rem","xnor","ror","rol","sla","sll"
 };
 
 // predefined attributes
-const char* g_vhdlKeyWordMap3[] =
+static const std::set< std::string > g_vhdlKeyWordSet3 =
 {
-"base","left","right","high","low","ascending",
-"image","value","pos","val","succ","pred","leftof","rightof","left","right","high","low",
-"range","reverse_range","length","ascending","delayed","stable","quiet","transaction","event",
-"active","last_event","last_active","last_value","driving","driving_value","simple_name","instance_name","path_name",0
+  "base","left","right","high","low","ascending",
+  "image","value","pos","val","succ","pred","leftof","rightof","left","right","high","low",
+  "range","reverse_range","length","ascending","delayed","stable","quiet","transaction","event",
+  "active","last_event","last_active","last_value","driving","driving_value","simple_name","instance_name","path_name"
 };
 
-  int j=0;
-  g_vhdlKeyDict0.setAutoDelete(TRUE);
-  g_vhdlKeyDict1.setAutoDelete(TRUE);
-  g_vhdlKeyDict2.setAutoDelete(TRUE);
-  g_vhdlKeyDict3.setAutoDelete(TRUE);
-
-  while (g_vhdlKeyWordMap0[j])
-  {
-    g_vhdlKeyDict0.insert(g_vhdlKeyWordMap0[j],
-	               new QCString(g_vhdlKeyWordMap0[j]));
-    j++;
-  }
-
-  j=0;
-  while (g_vhdlKeyWordMap1[j])
-  {
-    g_vhdlKeyDict1.insert(g_vhdlKeyWordMap1[j],
-	               new QCString(g_vhdlKeyWordMap1[j]));
-    j++;
-  }
-
-  j=0;
-  while (g_vhdlKeyWordMap2[j])
-  {
-    g_vhdlKeyDict2.insert(g_vhdlKeyWordMap2[j],
-	               new QCString(g_vhdlKeyWordMap2[j]));
-    j++;
-  }
-
-  j=0;
-  while (g_vhdlKeyWordMap3[j])
-  {
-    g_vhdlKeyDict3.insert(g_vhdlKeyWordMap3[j],
-	               new QCString(g_vhdlKeyWordMap3[j]));
-    j++;
-  }
-
-}// buildKeyMap
+void VhdlDocGen::init()
+{
+}
 
 /*!
  * returns the color of a keyword
  */
-
-QCString* VhdlDocGen::findKeyWord(const QCString& tmp)
+const char* VhdlDocGen::findKeyWord(const QCString& kw)
 {
-  static  QCString vhdlkeyword("vhdlkeyword");
-  static  QCString vhdltype("keywordtype");
-  static  QCString vhdllogic("vhdllogic");
-  static  QCString preprocessor("keywordflow");
+  std::string word=kw.lower().str();
 
-  QCString word=tmp.lower();
+  if (word.empty()) return 0;
 
-  if (word.isEmpty() || word.at(0)=='\0') return 0;
+  if (g_vhdlKeyWordSet0.find(word)!=g_vhdlKeyWordSet0.end())
+    return "keywordflow";
 
-  if (g_vhdlKeyDict0.find(word))
-    return &preprocessor;
+  if (g_vhdlKeyWordSet1.find(word)!=g_vhdlKeyWordSet1.end())
+    return "keywordtype";
 
-  if (g_vhdlKeyDict1.find(word))
-    return &vhdltype;
+  if (g_vhdlKeyWordSet2.find(word)!=g_vhdlKeyWordSet2.end())
+    return "vhdllogic";
 
-  if (g_vhdlKeyDict2.find(word))
-    return &vhdllogic;
-
-   if (g_vhdlKeyDict3.find(word))
-    return &vhdlkeyword;
+  if (g_vhdlKeyWordSet3.find(word)!=g_vhdlKeyWordSet3.end())
+    return "vhdlkeyword";
 
   return 0;
 }
@@ -693,26 +634,17 @@ QCString* VhdlDocGen::findKeyWord(const QCString& tmp)
 ClassDef *VhdlDocGen::getClass(const char *name)
 {
   if (name==0 || name[0]=='\0') return 0;
-
-  ClassDef *cd=0;
-  QCString temp(name);
-  //temp=temp.lower();
-  temp=temp.stripWhiteSpace();
-  cd= Doxygen::classSDict->find(temp.data());
-  return cd;
+  return Doxygen::classLinkedMap->find(QCString(name).stripWhiteSpace());
 }
 
 ClassDef* VhdlDocGen::getPackageName(const QCString & name)
 {
-  ClassDef* cd=0;
-  cd=getClass(name);
-
-  return cd;
+  return getClass(name);
 }
 
-static QMap<QCString,MemberDef*> varMap;
-static QList<ClassDef> qli;
-static QMap<ClassDef*,QList<ClassDef> > packages;
+static std::map<std::string,MemberDef*>            g_varMap;
+static std::vector<ClassDef*>                      g_classList;
+static std::map<ClassDef*,std::vector<ClassDef*> > g_packages;
 
 MemberDef* VhdlDocGen::findMember(const QCString& className, const QCString& memName)
 {
@@ -780,7 +712,7 @@ MemberDef* VhdlDocGen::findMember(const QCString& className, const QCString& mem
     }
     if (acd) //d && d->definitionType()==Definition::TypeClass)
     {
-      if(!packages.contains(acd))
+      if(g_packages.find(acd)==g_packages.end())
       {
         VhdlDocGen::findAllPackages(acd);
       }
@@ -789,20 +721,19 @@ MemberDef* VhdlDocGen::findMember(const QCString& className, const QCString& mem
   else
   {
     ecd=cd;
-    if (!packages.contains(ecd)) VhdlDocGen::findAllPackages(ecd);
+    if (g_packages.find(ecd)==g_packages.end()) VhdlDocGen::findAllPackages(ecd);
   }
 
   if (ecd)
   {
-    QMap<ClassDef*,QList<ClassDef> >::Iterator cList=packages.find(ecd);
-    if (cList!=packages.end())
+    auto cList_it = g_packages.find(ecd);
+    if (cList_it!=g_packages.end())
     {
-      QList<ClassDef> mlist=cList.data();
-      for (uint j=0;j<mlist.count();j++)
+      for (const auto &cdp : cList_it->second)
       {
-        mdef=VhdlDocGen::findMemberDef(mlist.at(j),memName,MemberListType_variableMembers);
+        mdef=VhdlDocGen::findMemberDef(cdp,memName,MemberListType_variableMembers);
         if (mdef) return mdef;
-        mdef=VhdlDocGen::findMemberDef(mlist.at(j),memName,MemberListType_pubMethods);
+        mdef=VhdlDocGen::findMemberDef(cdp,memName,MemberListType_pubMethods);
         if (mdef) return mdef;
       }
     }
@@ -822,21 +753,21 @@ MemberDef* VhdlDocGen::findMemberDef(ClassDef* cd,const QCString& key,MemberList
   QCString keyType=cd->symbolName()+"@"+key;
   //printf("\n %s | %s | %s",cd->symbolName().data(),key.data(,),keyType.data());
 
-  QMap<QCString, MemberDef*>::Iterator it =varMap.find(keyType);
-  if (it.key())
+  auto it = g_varMap.find(keyType.str());
+  if (it!=g_varMap.end())
   {
-    md=it.data();
+    md=it->second;
     if (md)
     {
       return md;
     }
   }
-  if (qli.contains(cd))
+  if (std::find(g_classList.begin(),g_classList.end(),cd)!=g_classList.end())
   {
     return 0;
   }
   ml=cd->getMemberList(type);
-  qli.append(cd);
+  g_classList.push_back(cd);
   if (!ml)
   {
     return 0;
@@ -848,16 +779,15 @@ MemberDef* VhdlDocGen::findMemberDef(ClassDef* cd,const QCString& key,MemberList
   for (fmni.toFirst();(md=fmni.current());++fmni)
   {
     QCString tkey=cd->symbolName()+"@"+md->name();
-    if (varMap.contains(tkey))
+    if (g_varMap.find(tkey.str())==g_varMap.end())
     {
-      continue;
+      g_varMap.insert({tkey.str(),md});
     }
-    varMap.insert(tkey.data(),md);
   }
-  it=varMap.find(keyType.data());
-  if (it.key())
+  it=g_varMap.find(keyType.str());
+  if (it!=g_varMap.end())
   {
-    md=it.data();
+    md=it->second;
     if (md)
     {
       return md;
@@ -872,8 +802,8 @@ MemberDef* VhdlDocGen::findMemberDef(ClassDef* cd,const QCString& key,MemberList
 
 void VhdlDocGen::findAllPackages( ClassDef *cdef)
 {
-  QList<ClassDef> cList;
-  if (packages.contains(cdef)) return;
+  if (g_packages.find(cdef)!=g_packages.end()) return;
+  std::vector<ClassDef*> cList;
   MemberList *mem=cdef->getMemberList(MemberListType_variableMembers);
   MemberDef *md;
 
@@ -887,9 +817,9 @@ void VhdlDocGen::findAllPackages( ClassDef *cdef)
       ClassDef* cd=VhdlDocGen::getPackageName(md->name());
       if (cd)
       {
-        cList.append(cd);
+        cList.push_back(cd);
         VhdlDocGen::findAllPackages(cd);
-        packages.insert(cdef,cList);
+        g_packages.insert({cdef,cList});
       }
     }
   }//for
@@ -1039,12 +969,10 @@ void VhdlDocGen::writeInlineClassLink(const ClassDef* cd ,OutputList& ol)
  */
 void VhdlDocGen::findAllArchitectures(QList<QCString>& qll,const ClassDef *cd)
 {
-  ClassDef *citer;
-  ClassSDict::Iterator cli(*Doxygen::classSDict);
-  for ( ; (citer=cli.current()) ; ++cli )
+  for (const auto &citer : *Doxygen::classLinkedMap)
   {
     QCString jj=citer->className();
-    if (cd != citer && jj.contains('-')!=-1)
+    if (cd != citer.get() && jj.contains('-')!=-1)
     {
       QCStringList ql=QCStringList::split("-",jj);
       QCString temp=ql[1];
@@ -1057,13 +985,10 @@ void VhdlDocGen::findAllArchitectures(QList<QCString>& qll,const ClassDef *cd)
   }// for
 }//findAllArchitectures
 
-ClassDef* VhdlDocGen::findArchitecture(const ClassDef *cd)
+const ClassDef* VhdlDocGen::findArchitecture(const ClassDef *cd)
 {
-  ClassDef *citer;
   QCString nn=cd->name();
-  ClassSDict::Iterator cli(*Doxygen::classSDict);
-
-  for ( ; (citer=cli.current()) ; ++cli )
+  for (const auto &citer : *Doxygen::classLinkedMap)
   {
     QCString jj=citer->name();
     QCStringList ql=QCStringList::split(":",jj);
@@ -1071,7 +996,7 @@ ClassDef* VhdlDocGen::findArchitecture(const ClassDef *cd)
     {
       if (ql[0]==nn )
       {
-        return citer;
+        return citer.get();
       }
     }
   }
@@ -1272,7 +1197,6 @@ void VhdlDocGen::writeFormatString(const QCString& s,OutputList&ol,const MemberD
   QRegExp reg("[\\[\\]\\.\\/\\:\\<\\>\\:\\s\\,\\;\\'\\+\\-\\*\\|\\&\\=\\(\\)\"]");
   QCString qcs = s;
   qcs+=QCString(" ");// parsing the last sign
-  QCString *ss;
   QCString find=qcs;
   QCString temp=qcs;
   char buf[2];
@@ -1289,7 +1213,7 @@ void VhdlDocGen::writeFormatString(const QCString& s,OutputList&ol,const MemberD
     {
       find=find.left(j);
       buf[0]=temp[j];
-      ss=VhdlDocGen::findKeyWord(find);
+      const char *ss=VhdlDocGen::findKeyWord(find);
       bool k=isNumber(find); // is this a number
       if (k)
       {
@@ -1299,7 +1223,7 @@ void VhdlDocGen::writeFormatString(const QCString& s,OutputList&ol,const MemberD
       }
       else if (j != 0 && ss)
       {
-        startFonts(find,ss->data(),ol);
+        startFonts(find,ss,ol);
       }
       else
       {
@@ -1435,11 +1359,11 @@ void VhdlDocGen::writeProcedureProto(OutputList& ol,const ArgumentList &al,const
     nn+=": ";
 
     QCString defval = arg.defval;
-    QCString *str=VhdlDocGen::findKeyWord(defval);
+    const char *str=VhdlDocGen::findKeyWord(defval);
     defval+=" ";
     if (str)
     {
-      startFonts(defval,str->data(),ol);
+      startFonts(defval,str,ol);
     }
     else
     {
@@ -1500,7 +1424,7 @@ void VhdlDocGen::writeFunctionProto(OutputList& ol,const ArgumentList &al,const
     }
     if (!att.isEmpty())
     {
-      QCString *str=VhdlDocGen::findKeyWord(att);
+      const char *str=VhdlDocGen::findKeyWord(att);
       att+=" ";
       if (str)
         VhdlDocGen::formatString(att,ol,mdef);
@@ -1514,7 +1438,7 @@ void VhdlDocGen::writeFunctionProto(OutputList& ol,const ArgumentList &al,const
     QCString w=ss.stripWhiteSpace();//.upper();
     startFonts(nn,"vhdlchar",ol);
     startFonts("in ","stringliteral",ol);
-    QCString *str=VhdlDocGen::findKeyWord(ss);
+    const char *str=VhdlDocGen::findKeyWord(ss);
     if (str)
       VhdlDocGen::formatString(w,ol,mdef);
     else
@@ -1717,7 +1641,7 @@ void VhdlDocGen::writeVhdlDeclarations(const MemberList* ml,
 
 }
 
-void VhdlDocGen::correctMemberProperties(MemberDef *md)
+void VhdlDocGen::correctMemberProperties(MemberDefMutable *md)
 {
   if (qstrcmp(md->argsString(),"package")==0)
   {
@@ -1767,7 +1691,7 @@ void VhdlDocGen::correctMemberProperties(MemberDef *md)
 /* writes a vhdl type documentation */
 bool VhdlDocGen::writeVHDLTypeDocumentation(const MemberDef* mdef, const Definition *d, OutputList &ol)
 {
-  const ClassDef *cd=dynamic_cast<const ClassDef*>(d);
+  const ClassDef *cd=toClassDef(d);
   bool hasParams = FALSE;
 
   if (cd==0) return hasParams;
@@ -1849,7 +1773,7 @@ bool VhdlDocGen::writeVHDLTypeDocumentation(const MemberDef* mdef, const Definit
   return hasParams;
 }
 
-void VhdlDocGen::writeTagFile(MemberDef *mdef,FTextStream &tagFile)
+void VhdlDocGen::writeTagFile(MemberDefMutable *mdef,FTextStream &tagFile)
 {
   tagFile << "    <member kind=\"";
   if (VhdlDocGen::isGeneric(mdef))      tagFile << "generic";
@@ -1893,7 +1817,7 @@ void VhdlDocGen::writeTagFile(MemberDef *mdef,FTextStream &tagFile)
 
 /* writes a vhdl type declaration */
 
-void VhdlDocGen::writeVHDLDeclaration(const MemberDef* mdef,OutputList &ol,
+void VhdlDocGen::writeVHDLDeclaration(const MemberDefMutable* mdef,OutputList &ol,
     const ClassDef *cd,const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,
     bool /*inGroup*/)
 {
@@ -2199,32 +2123,34 @@ void VhdlDocGen::writePlainVHDLDeclarations(
     const ClassDef *cd,const NamespaceDef *nd,const FileDef *fd,const GroupDef *gd,int specifier)
 {
 
-  SDict<QCString> pack(1009);
-  pack.setAutoDelete(TRUE);
+  StringSet pack;
 
   bool first=TRUE;
-  MemberDef *md;
+  MemberDef *imd;
   MemberListIterator mli(*mlist);
-  for ( ; (md=mli.current()); ++mli )
+  for ( ; (imd=mli.current()); ++mli )
   {
-    int mems=md->getMemberSpecifiers();
-    if (md->isBriefSectionVisible() && (mems==specifier) && (mems!=VhdlDocGen::LIBRARY) )
+    MemberDefMutable *md = toMemberDefMutable(imd);
+    if (md)
     {
-      if (first) { ol.startMemberList();first=FALSE; }
-      VhdlDocGen::writeVHDLDeclaration(md,ol,cd,nd,fd,gd,FALSE);
-    } //if
-    else if (md->isBriefSectionVisible() && (mems==specifier))
-    {
-      if (!pack.find(md->name().data()))
+      int mems=md->getMemberSpecifiers();
+      if (md->isBriefSectionVisible() && (mems==specifier) && (mems!=VhdlDocGen::LIBRARY) )
       {
-        if (first) ol.startMemberList(),first=FALSE;
+        if (first) { ol.startMemberList();first=FALSE; }
         VhdlDocGen::writeVHDLDeclaration(md,ol,cd,nd,fd,gd,FALSE);
-        pack.append(md->name().data(),new QCString(md->name().data()));
-      }
+      } //if
+      else if (md->isBriefSectionVisible() && (mems==specifier))
+      {
+        if (pack.find(md->name().str())==pack.end())
+        {
+          if (first) ol.startMemberList(),first=FALSE;
+          VhdlDocGen::writeVHDLDeclaration(md,ol,cd,nd,fd,gd,FALSE);
+          pack.insert(md->name().str());
+        }
+      } //if
     } //if
   } //for
   if (!first) ol.endMemberList();
-  pack.clear();
 }//plainDeclaration
 
 static bool membersHaveSpecificType(const MemberList *ml,uint64 type)
@@ -2239,17 +2165,11 @@ static bool membersHaveSpecificType(const MemberList *ml,uint64 type)
       return TRUE;
     }
   }
-  if (ml->getMemberGroupList())
+  for (const auto &mg : ml->getMemberGroupList())
   {
-    MemberGroupListIterator mgli(*ml->getMemberGroupList());
-    MemberGroup *mg;
-    while ((mg=mgli.current()))
+    if (mg->members())
     {
-      if (mg->members())
-      {
-        if (membersHaveSpecificType(mg->members(),type)) return TRUE;
-      }
-      ++mgli;
+      if (membersHaveSpecificType(mg->members(),type)) return TRUE;
     }
   }
   return FALSE;
@@ -2278,36 +2198,30 @@ void VhdlDocGen::writeVHDLDeclarations(const MemberList* ml,OutputList &ol,
 
   VhdlDocGen::writePlainVHDLDeclarations(ml,ol,cd,nd,fd,gd,type);
 
-  if (ml->getMemberGroupList())
+  for (const auto &mg : ml->getMemberGroupList())
   {
-    MemberGroupListIterator mgli(*ml->getMemberGroupList());
-    MemberGroup *mg;
-    while ((mg=mgli.current()))
+    if (membersHaveSpecificType(mg->members(),type))
     {
-      if (membersHaveSpecificType(mg->members(),type))
+      //printf("mg->header=%s\n",mg->header().data());
+      bool hasHeader=mg->header()!="[NOHEADER]";
+      ol.startMemberGroupHeader(hasHeader);
+      if (hasHeader)
       {
-        //printf("mg->header=%s\n",mg->header().data());
-        bool hasHeader=mg->header()!="[NOHEADER]";
-        ol.startMemberGroupHeader(hasHeader);
-        if (hasHeader)
-        {
-          ol.parseText(mg->header());
-        }
-        ol.endMemberGroupHeader();
-        if (!mg->documentation().isEmpty())
-        {
-          //printf("Member group has docs!\n");
-          ol.startMemberGroupDocs();
-          ol.generateDoc("[generated]",-1,0,0,mg->documentation()+"\n",FALSE,FALSE,
-                         0,FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
-          ol.endMemberGroupDocs();
-        }
-        ol.startMemberGroup();
-        //printf("--- mg->writePlainDeclarations ---\n");
-        VhdlDocGen::writePlainVHDLDeclarations(mg->members(),ol,cd,nd,fd,gd,type);
-        ol.endMemberGroup(hasHeader);
+        ol.parseText(mg->header());
       }
-      ++mgli;
+      ol.endMemberGroupHeader();
+      if (!mg->documentation().isEmpty())
+      {
+        //printf("Member group has docs!\n");
+        ol.startMemberGroupDocs();
+        ol.generateDoc("[generated]",-1,0,0,mg->documentation()+"\n",FALSE,FALSE,
+            0,FALSE,FALSE,Config_getBool(MARKDOWN_SUPPORT));
+        ol.endMemberGroupDocs();
+      }
+      ol.startMemberGroup();
+      //printf("--- mg->writePlainDeclarations ---\n");
+      VhdlDocGen::writePlainVHDLDeclarations(mg->members(),ol,cd,nd,fd,gd,type);
+      ol.endMemberGroup(hasHeader);
     }
   }
 }// writeVHDLDeclarations
@@ -2354,9 +2268,9 @@ void VhdlDocGen::writeStringLink(const MemberDef *mdef,QCString mem, OutputList&
 
 
 
-void VhdlDocGen::writeSource(const MemberDef *mdef,OutputList& ol,const QCString & cname)
+void VhdlDocGen::writeSource(const MemberDefMutable *mdef,OutputList& ol,const QCString & cname)
 {
-  CodeParserInterface &intf = Doxygen::parserManager->getCodeParser(".vhd");
+  auto intf = Doxygen::parserManager->getCodeParser(".vhd");
  // pIntf->resetCodeParserState();
 
   QCString codeFragment=mdef->documentation();
@@ -2381,8 +2295,8 @@ void VhdlDocGen::writeSource(const MemberDef *mdef,OutputList& ol,const QCString
 
   codeFragment.prepend("\n");
   ol.pushGeneratorState();
-  ol.startCodeFragment();
-  intf.parseCode(      ol,               // codeOutIntf
+  ol.startCodeFragment("DoxyCode");
+  intf->parseCode(     ol,               // codeOutIntf
                        0,                // scope
                        codeFragment,     // input
                        SrcLangExt_VHDL,  // lang
@@ -2396,7 +2310,7 @@ void VhdlDocGen::writeSource(const MemberDef *mdef,OutputList& ol,const QCString
                        TRUE              // show line numbers
                       );
 
-  ol.endCodeFragment();
+  ol.endCodeFragment("DoxyCode");
   ol.popGeneratorState();
 
   if (cname.isEmpty()) return;
@@ -2632,21 +2546,18 @@ QCString  VhdlDocGen::parseForBinding(QCString & entity,QCString & arch)
 
 
 
- // find class with upper/lower letters
- ClassDef* VhdlDocGen::findVhdlClass(const char *className )
+// find class with upper/lower letters
+ClassDef* VhdlDocGen::findVhdlClass(const char *className )
+{
+ for (const auto &cd : *Doxygen::classLinkedMap)
  {
-
-  ClassSDict::Iterator cli(*Doxygen::classSDict);
-  ClassDef *cd;
-  for (;(cd=cli.current());++cli)
-  {
-    if (qstricmp(className,cd->name().data())==0)
-    {
-      return cd;
-    }
-  }
-  return 0;
+   if (qstricmp(className,cd->name().data())==0)
+   {
+     return cd.get();
+   }
  }
+ return 0;
+}
 
 
 /*
@@ -2684,10 +2595,10 @@ void VhdlDocGen::computeVhdlComponentRelations()
       entity=cur->type;
     }
 
-    ClassDef *classEntity= VhdlDocGen::findVhdlClass(entity.data());//Doxygen::classSDict->find(entity);
+    ClassDefMutable *classEntity= toClassDefMutable(VhdlDocGen::findVhdlClass(entity.data()));
     inst=VhdlDocGen::getIndexWord(cur->args.data(),0);
-    ClassDef *cd=Doxygen::classSDict->find(inst);
-    ClassDef *ar=Doxygen::classSDict->find(cur->args);
+    ClassDefMutable *cd=toClassDefMutable(Doxygen::classLinkedMap->find(inst));
+    ClassDefMutable *ar=toClassDefMutable(Doxygen::classLinkedMap->find(cur->args));
 
     if (cd==0)
     {
@@ -2702,8 +2613,8 @@ void VhdlDocGen::computeVhdlComponentRelations()
 
 }
 
-static void addInstance(ClassDef* classEntity, ClassDef* ar,
-                        ClassDef *cd , const std::shared_ptr<Entry> &cur)
+static void addInstance(ClassDefMutable* classEntity, ClassDefMutable* ar,
+                        ClassDefMutable *cd , const std::shared_ptr<Entry> &cur)
 {
 
   QCString bName,n1;
@@ -2739,14 +2650,14 @@ static void addInstance(ClassDef* classEntity, ClassDef* ar,
 
 ferr:
   QCString uu=cur->name;
-  MemberDef *md=createMemberDef(
+  std::unique_ptr<MemberDefMutable> md { createMemberDef(
       ar->getDefFileName(), cur->startLine,cur->startColumn,
       n1,uu,uu, 0,
       Public, Normal, cur->stat,Member,
       MemberType_Variable,
       ArgumentList(),
       ArgumentList(),
-      "");
+      "") };
 
   if (ar->getOutputFileBase())
   {
@@ -2774,27 +2685,29 @@ ferr:
   //label.replace(epr,":");
   //info+=label;
   //fprintf(stderr,"\n[%s:%d:%s]\n",fd->fileName().data(),cur->startLine,info.data());
-  ar->insertMember(md);
+  ar->insertMember(md.get());
+  MemberName *mn = Doxygen::functionNameLinkedMap->add(uu);
+  mn->push_back(std::move(md));
 
 }
 
 
-void  VhdlDocGen::writeRecordUnit(QCString & largs,QCString & ltype,OutputList& ol ,const MemberDef *mdef)
+void  VhdlDocGen::writeRecordUnit(QCString & largs,QCString & ltype,OutputList& ol ,const MemberDefMutable *mdef)
 {
-	  int i=mdef->name().find('~');
-	  if(i>0){
-		 //sets the real record member name
-	    const_cast<MemberDef*>(mdef)->setName(mdef->name().left(i).data());
-       }
-
-      writeLink(mdef,ol);
-      ol.startBold();
-      ol.insertMemberAlign();
-      if (!ltype.isEmpty()){
-        VhdlDocGen::formatString(ltype,ol,mdef);
-	}
-      ol.endBold();
+  int i=mdef->name().find('~');
+  if (i>0)
+  {
+    //sets the real record member name
+    const_cast<MemberDefMutable*>(mdef)->setName(mdef->name().left(i).data());
+  }
 
+  writeLink(mdef,ol);
+  ol.startBold();
+  ol.insertMemberAlign();
+  if (!ltype.isEmpty()){
+    VhdlDocGen::formatString(ltype,ol,mdef);
+  }
+  ol.endBold();
 }
 
 
@@ -2843,24 +2756,20 @@ bool VhdlDocGen::isSubClass(ClassDef* cd,ClassDef *scd, bool followInstances,int
     return FALSE;
   }
 
-  if (cd->subClasses())
+  for (const auto &bcd :cd->subClasses())
   {
-    BaseClassListIterator bcli(*cd->subClasses());
-    for ( ; bcli.current() && !found ; ++bcli)
+    const ClassDef *ccd=bcd.classDef;
+    if (!followInstances && ccd->templateMaster()) ccd=ccd->templateMaster();
+    //printf("isSubClass() subclass %s\n",ccd->name().data());
+    if (ccd==scd)
     {
-      const ClassDef *ccd=bcli.current()->classDef;
-      if (!followInstances && ccd->templateMaster()) ccd=ccd->templateMaster();
-      //printf("isSubClass() subclass %s\n",ccd->name().data());
-      if (ccd==scd)
+      found=TRUE;
+    }
+    else
+    {
+      if (level <256)
       {
-        found=TRUE;
-      }
-      else
-      {
-        if (level <256)
-        {
-          found=ccd->isBaseClass(scd,followInstances,level+1);
-        }
+        found=ccd->isBaseClass(scd,followInstances,level+1);
       }
     }
   }
@@ -2869,35 +2778,33 @@ bool VhdlDocGen::isSubClass(ClassDef* cd,ClassDef *scd, bool followInstances,int
 
 void VhdlDocGen::addBaseClass(ClassDef* cd,ClassDef *ent)
 {
-  if (cd->baseClasses())
+  BaseClassList bcl = cd->baseClasses();
+  for (auto &bcd : bcl)
   {
-    BaseClassListIterator bcli(*cd->baseClasses());
-    for ( ; bcli.current()  ; ++bcli)
+    ClassDef *ccd = bcd.classDef;
+    if (ccd==ent)
     {
-      ClassDef *ccd=bcli.current()->classDef;
-      if (ccd==ent)
+      QCString n = bcd.usedName;
+      int i = n.find('(');
+      if(i<0)
       {
-        QCString n = bcli.current()->usedName;
-        int i = n.find('(');
-        if(i<0)
-        {
-          bcli.current()->usedName.append("(2)");
-          return;
-        }
-        static QRegExp reg("[0-9]+");
-        QCString s=n.left(i);
-        QCString r=n.right(n.length()-i);
-        QCString t=r;
-        VhdlDocGen::deleteAllChars(r,')');
-        VhdlDocGen::deleteAllChars(r,'(');
-        r.setNum(r.toInt()+1);
-        t.replace(reg,r.data());
-        s.append(t.data());
-        bcli.current()->usedName=s;
-        bcli.current()->templSpecifiers=t;
+        bcd.usedName.append("(2)");
+        return;
       }
+      static QRegExp reg("[0-9]+");
+      QCString s=n.left(i);
+      QCString r=n.right(n.length()-i);
+      QCString t=r;
+      VhdlDocGen::deleteAllChars(r,')');
+      VhdlDocGen::deleteAllChars(r,'(');
+      r.setNum(r.toInt()+1);
+      t.replace(reg,r.data());
+      s.append(t.data());
+      bcd.usedName=s;
+      bcd.templSpecifiers=t;
     }
   }
+  cd->updateBaseClasses(bcl);
 }
 
 
@@ -2948,9 +2855,9 @@ void VhdlDocGen::createFlowChart(const MemberDef *mdef)
 
 void VhdlDocGen::resetCodeVhdlParserState()
 {
-  varMap.clear();
-  qli.clear();
-  packages.clear();
+  g_varMap.clear();
+  g_classList.clear();
+  g_packages.clear();
 }
 
 bool VhdlDocGen::isConstraint(const MemberDef *mdef)
@@ -3061,7 +2968,7 @@ static struct
 QList<FlowChart>  FlowChart::flowList;
 
 #ifdef DEBUGFLOW
-static QMap<QCString,int> keyMap;
+static std::map<std::string,int> g_keyMap;
 #endif
 
 void alignText(QCString & q)
@@ -3683,7 +3590,7 @@ void FlowChart::writeShape(FTextStream &t,const FlowChart* fl)
 
 #ifdef DEBUGFLOW
   QCString qq(getNodeName(fl->id).data());
-  keyMap.insert(qq,fl->id);
+  g_keyMap.insert({qq.str(),fl->id});
 #endif
 
   bool dec=(fl->type & DECLN);
@@ -3772,11 +3679,11 @@ void FlowChart::writeEdge(FTextStream &t,const FlowChart* fl_from,const FlowChar
 #ifdef DEBUGFLOW
   QCString s1(getNodeName(fl_from->id).data());
   QCString s2(getNodeName(fl_to->id).data());
-  QMap<QCString, int>::Iterator it = keyMap.find(s1);
-  QMap<QCString, int>::Iterator it1 = keyMap.find(s2);
+  auto it = g_keyMap.find(s1.str());
+  auto it1 = g_keyMap.find(s2.str());
   // checks if the link is connected to a valid node
-  assert(it.key());
-  assert(it1.key());
+  assert(it!=g_keyMap.end());
+  assert(it1!=g_keyMap.end());
 #endif
 
   writeEdge(t,fl_from->id,fl_to->id,i,b,c);
