@@ -41,7 +41,7 @@ static void visitCaption(XmlDocVisitor *parent, QList<DocNode> children)
 static void visitPreStart(FTextStream &t, const char *cmd, bool doCaption,
                           XmlDocVisitor *parent, QList<DocNode> children,
                           const QCString &name, bool writeType, DocImage::Type type, const QCString &width,
-                          const QCString &height, bool inlineImage = FALSE)
+                          const QCString &height, const QCString &alt = QCString(""), bool inlineImage = FALSE)
 {
   t << "<" << cmd;
   if (writeType)
@@ -68,6 +68,10 @@ static void visitPreStart(FTextStream &t, const char *cmd, bool doCaption,
   {
     t << " height=\"" << convertToXML(height) << "\"";
   }
+  if (!alt.isEmpty())
+  {
+    t << " alt=\"" << convertToXML(alt) << "\"";
+  }
   if (inlineImage)
   {
     t << " inline=\"yes\"";
@@ -252,9 +256,8 @@ void XmlDocVisitor::visit(DocVerbatim *s)
           m_t << " filename=\"" << lang << "\">";
       else
           m_t << ">";
-      Doxygen::parserManager->getCodeParser(lang)
-                             .parseCode(m_ci,s->context(),s->text(),langExt,
-                                        s->isExample(),s->exampleFile());
+      getCodeParser(lang).parseCode(m_ci,s->context(),s->text(),langExt,
+                                    s->isExample(),s->exampleFile());
       m_t << "</programlisting>";
       break;
     case DocVerbatim::Verbatim:
@@ -332,8 +335,7 @@ void XmlDocVisitor::visit(DocInclude *inc)
          m_t << "<programlisting filename=\"" << inc->file() << "\">";
          QFileInfo cfi( inc->file() );
          FileDef *fd = createFileDef( cfi.dirPath().utf8(), cfi.fileName().utf8() );
-         Doxygen::parserManager->getCodeParser(inc->extension())
-                                .parseCode(m_ci,inc->context(),
+         getCodeParser(inc->extension()).parseCode(m_ci,inc->context(),
                                            inc->text(),
                                            langExt,
                                            inc->isExample(),
@@ -351,8 +353,7 @@ void XmlDocVisitor::visit(DocInclude *inc)
       break;
     case DocInclude::Include:
       m_t << "<programlisting filename=\"" << inc->file() << "\">";
-      Doxygen::parserManager->getCodeParser(inc->extension())
-                             .parseCode(m_ci,inc->context(),
+      getCodeParser(inc->extension()).parseCode(m_ci,inc->context(),
                                         inc->text(),
                                         langExt,
                                         inc->isExample(),
@@ -411,8 +412,7 @@ void XmlDocVisitor::visit(DocInclude *inc)
       break;
     case DocInclude::Snippet:
       m_t << "<programlisting filename=\"" << inc->file() << "\">";
-      Doxygen::parserManager->getCodeParser(inc->extension())
-                             .parseCode(m_ci,
+      getCodeParser(inc->extension()).parseCode(m_ci,
                                         inc->context(),
                                         extractBlock(inc->text(),inc->blockId()),
                                         langExt,
@@ -426,8 +426,7 @@ void XmlDocVisitor::visit(DocInclude *inc)
          m_t << "<programlisting filename=\"" << inc->file() << "\">";
          QFileInfo cfi( inc->file() );
          FileDef *fd = createFileDef( cfi.dirPath().utf8(), cfi.fileName().utf8() );
-         Doxygen::parserManager->getCodeParser(inc->extension())
-                                .parseCode(m_ci,
+         getCodeParser(inc->extension()).parseCode(m_ci,
                                            inc->context(),
                                            extractBlock(inc->text(),inc->blockId()),
                                            langExt,
@@ -480,17 +479,16 @@ void XmlDocVisitor::visit(DocIncOperator *op)
         fd = createFileDef( cfi.dirPath().utf8(), cfi.fileName().utf8() );
       }
 
-      Doxygen::parserManager->getCodeParser(locLangExt)
-                             .parseCode(m_ci,op->context(),
-                                        op->text(),langExt,op->isExample(),
-                                        op->exampleFile(),
-                                        fd,     // fileDef
-                                        op->line(),    // startLine
-                                        -1,    // endLine
-                                        FALSE, // inline fragment
-                                        0,     // memberDef
-                                        op->showLineNo()  // show line numbers
-                                       );
+      getCodeParser(locLangExt).parseCode(m_ci,op->context(),
+                                          op->text(),langExt,op->isExample(),
+                                          op->exampleFile(),
+                                          fd,     // fileDef
+                                          op->line(),    // startLine
+                                          -1,    // endLine
+                                          FALSE, // inline fragment
+                                          0,     // memberDef
+                                          op->showLineNo()  // show line numbers
+                                         );
       if (fd) delete fd;
     }
     pushEnabled();
@@ -780,7 +778,17 @@ void XmlDocVisitor::visitPre(DocHtmlTable *t)
 {
   if (m_hide) return;
   m_t << "<table rows=\"" << t->numRows()
-      << "\" cols=\"" << t->numColumns() << "\">" ;
+      << "\" cols=\"" << t->numColumns() << "\"" ;
+  HtmlAttribListIterator li(t->attribs());
+  HtmlAttrib* opt;
+  for (li.toFirst(); (opt = li.current()); ++li)
+  {
+    if (opt->name=="width")
+    {
+      m_t << " " << opt->name << "=\"" << opt->value << "\"";
+    }
+  }
+  m_t << ">";
 }
 
 void XmlDocVisitor::visitPost(DocHtmlTable *)
@@ -818,6 +826,15 @@ void XmlDocVisitor::visitPre(DocHtmlCell *c)
     {
       m_t << " align=\"" << opt->value << "\"";
     }
+    else if (opt->name=="valign" &&
+      (opt->value == "bottom" || opt->value == "top" || opt->value == "middle"))
+    {
+      m_t << " valign=\"" << opt->value << "\"";
+    }
+    else if (opt->name=="width")
+    {
+      m_t << " width=\"" << opt->value << "\"";
+    }
     else if (opt->name=="class") // handle markdown generated attributes
     {
       if (opt->value.left(13)=="markdownTable") // handle markdown generated attributes
@@ -913,7 +930,7 @@ void XmlDocVisitor::visitPre(DocImage *img)
   {
     baseName = correctURL(url,img->relPath());
   }
-  visitPreStart(m_t, "image", FALSE, this, img->children(), baseName, TRUE, img->type(), img->width(), img->height(), img ->isInlineImage());
+  visitPreStart(m_t, "image", FALSE, this, img->children(), baseName, TRUE, img->type(), img->width(), img->height(), img->attribs().find("alt"), img->isInlineImage());
 
   // copy the image to the output dir
   FileDef *fd;
